/**
\class NonlinearPIDController
\par User Documentation:

\par Parameters
<table >
<tr><td><b>Parameter<td>Size<td>Comment</b>
<tr><td>SaturationFactor_TauIntegral<td>6<td>Factor indicating how large percentage the integral effect should be given of the available force
<tr><td>RigidBodyandAddedMassMatrixDiagonal<td>6<td>Rigid body and added mass matrix diagonal
<tr><td>LinearDampingMatrixDiagonal<td>6<td>Linear damping matrix diagonal
<tr><td>NonlinearDampingMatrixDiagonal<td>6<td>quadratic damping diagonal
<tr><td>IntegralLimits<td>6<td>limits for when the integrator starts to integrate
<tr><td>AntiWindupConstants<td>6<td>Constants for the anti windup effect
<tr><td>TauSaturationLimitMin<td>6<td>Minimum available force in all 6DOF
<tr><td>TauSaturationLimitMax<td>6<td>Maximum available force in all 6DOF
</table>

*/
#ifndef NONLINEARPIDCONTROLLER_H
#define NONLINEARPIDCONTROLLER_H

#include "Eigen\Dense"
#include <sfhGeneral.h>
#include "PIDController.h"

class NonlinearPIDController : public PIDController
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	NonlinearPIDController(const string& simObjectName, ISimObjectCreator* const creator); ///< reads parameters, registers states, input/output ports and shared resources
	const double* const ForcesAndMomentsBODY(const double T, const double* const X);		///< Returns calculated forces and moments
	void OdeFcn(const double T, const double* const X, double* const XDot, const bool bIsMajorTimeStep); ///< returns object derivatives as a function of time, states and input ports
	
#ifdef FH_VISUALIZATION
	/// The rendering is not used.
	virtual void RenderInit(Ogre::Root* const ogreRoot, ISimObjectCreator* const creator){}

	/// The rendering is not used.
	virtual void RenderUpdate(const double T, const double* const X){}
#endif		

private:
	void CalculateTauFF(const double T,const double* const X);			///< method for calculating the feed forward term
	void CalculateTauI(const double T,const double* const X);			///< mthod for calculating the integral term
	double Saturate(double value, double min, double max);				///< method for saturaiong
	double integralSaturationCoeffs[6];									///< coefficients for limiting changing saturated forces and moments
	double dampingLinearDiagonal[6];									///< array holding the diagonal elements in the linear damping matrix, for feed forward term
	double dampingNonlinearDiagonal[6];									///< array holding the diagonal elements in the quadratic damping matrix, for feed forward term
	double massDiagonal[6];												///< array holding the diagonal elements in the rigid body and added mass matrix, for feed forward term
	double a_tauFF[6];													///< return array for the feed forward term
	double antiWindupConstants[6];										///< gain constants for the anti windup effect
	
	double tauSatMin[6];												///< Lower saturation limit on forces and moments
	double tauSatMax[6];												///< Upper saturation limit on force and moments


#ifdef FH_VISUALIZATION
#endif

};
#endif