////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							SampledSignalWithNoise
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// 			Sampled the signals at a given sampling rate, and outputs as a ZOH signal. Adds colored noise to the signal
///
/// \author			Mats N{\aa}vik Hval
///
/// \par Revision history:
///					29.04.2012 MNH: Initial version.
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef SAMPLEDSIGNALWITHNOISE_H
#define SAMPLEDSIGNALWITHNOISE_H

#include "SimObject.h"
#include <Eigen\Dense>
#include "math.h"
#include <sfhGeneral.h>
#include <vector>
#include "HelpFunctions.h"

class SampledSignalWithNoise : public SimObject
{
public:
	SampledSignalWithNoise(const string& simObjectName, ISimObjectCreator* const creator); ///< reads parameters, registers states, input/output ports and shared resources
	~SampledSignalWithNoise();
	const double* const Signal(const double T,const double* const X); ///< returns the distorted signal
	void OdeFcn(const double T, const double* const X, double* const XDot, const bool bIsMajorTimeStep); ///< returns object derivatives as a function of time, states and input ports

#ifdef FH_VISUALIZATION
	void	RenderInit(Ogre::Root* const ogreRoot, ISimObjectCreator* const creator);
	void	RenderUpdate(const double T, const double* const X);
#endif


private:

	ISignalPort* signalIn;										///< input port of the smooth signal

	float randomDouble(double min, double max);					///< method to generate a random signal
	int numberOfSignals;										///< number of signals to deal with
	double *coloredSignals;										///< array of the distorted signal
	double *noiseAmplitude;										///< array of noise amplitudes
	double *signalSamplingFreq;									///< array of sampling frequencies
	double *signal;												///< array of signal

	int *count;													///< time step counter
	int *countReset;											///< signal is sampled at each countReset number of time steps
	std::vector<double> vec_noiseAmplitude;						///< vector of noise amplitude
	std::vector<double> vec_signalSamplingFrequency;			///< vector of sampling frequencies

};

#endif
