#include "ThrusterAllocation.h"

ThrusterAllocation::ThrusterAllocation(const string& simObjectName, ISimObjectCreator* const creator) : SimObject(simObjectName)
{
	
	creator->GetIntParam("NumberOfThrusters",&numberOfThrusters);
	Thr = Eigen::MatrixXd::Zero(6,numberOfThrusters);
	TpseudoInverse = Eigen::MatrixXd::Zero(6,numberOfThrusters);
	K = Eigen::MatrixXd::Zero(numberOfThrusters,numberOfThrusters);
	Kinv = Eigen::MatrixXd::Zero(numberOfThrusters,numberOfThrusters);
	u = Eigen::VectorXd::Zero(numberOfThrusters);
	tau = Eigen::Matrix<double,6,1>::Zero();
	tauCommand = Eigen::VectorXd::Zero(numberOfThrusters);
//	previousRPM = new int[numberOfThrusters];
	RPMvalues = new double[numberOfThrusters];
	for (size_t i = 0;i< numberOfThrusters;i++){
	//	previousRPM[i] = 0;
		RPMvalues[i] = 0;
		tauCommanded[i] = 0;
	}
	string AllocType = "fixedAttitude"; 
	creator->GetStringParam("AllocationType",allocationType,AllocType);
	if (allocationType.compare("fixedAttitude") == 0){
		string attributeName;
		for (size_t i = 0;i<numberOfThrusters;i++){
			string thrustCalculationBasis;
			creator->GetStringParam("ThrustCalculationBasis",thrustCalculationBasis,"secondOrder");
			if (thrustCalculationBasis.compare("linear") == 0 || thrustCalculationBasis.compare("Linear") == 0)
				linearRelation = true;
			else
				linearRelation = false;

			//ThrusterStruct *thruster = new ThrusterStruct();
			thrusters.push_back(ThrusterStruct());
			///**********************************
			/// Adds information of each thruster
			///**********************************
			attributeName = "Thruster_"+ intToString(i+1);
			attributeName.append("_positionAndAttitude");
			creator->GetDoubleParamArray(attributeName,thrusters.back().positionAndAttitude,5);
			
			attributeName = "Thruster_"+ intToString(i+1);
			attributeName.append("_RPMmin_RPMmax");
			creator->GetIntParamArray(attributeName,thrusters.back().RPMlimits,2);
		
			attributeName = "Thruster_"+ intToString(i+1);
			if (linearRelation){
				attributeName.append("_thrustCoeff");
				creator->GetDoubleParamArray(attributeName,thrusters.back().RPMthrustCoeffs,1);
				thrusters.back().RPMthrustCoeffs[0] = sqrt(thrusters.back().RPMthrustCoeffs[0]*thrusters.back().RPMlimits[1]);
				thrusters.back().RPMthrustCoeffs[1] = 0;
			}
			else{
				attributeName.append("_thrustCoeff_A_B");
				creator->GetDoubleParamArray(attributeName,thrusters.back().RPMthrustCoeffs,2);
			}
			
			thrusters.back().rpm = 0;
			Thr(0,i) = cos(thrusters.back().positionAndAttitude[3]*M_PI/180)*sin(thrusters.back().positionAndAttitude[4]*M_PI/180);
			Thr(1,i) = sin(thrusters.back().positionAndAttitude[3]*M_PI/180)*sin(thrusters.back().positionAndAttitude[4]*M_PI/180);
			Thr(2,i) = cos(thrusters.back().positionAndAttitude[4]*M_PI/180);
			Thr(3,i) = Thr(2,i)*thrusters.back().positionAndAttitude[1] - Thr(1,i)*thrusters.back().positionAndAttitude[2];
			Thr(4,i) = Thr(0,i)*thrusters.back().positionAndAttitude[2] - Thr(2,i)*thrusters.back().positionAndAttitude[0];
			Thr(5,i) = Thr(1,i)*thrusters.back().positionAndAttitude[0] - Thr(0,i)*thrusters.back().positionAndAttitude[1];
			//std::cout << Thr << std::endl << i << "------------" << std::endl;
		//	thrusters.push_back(thruster);
			//std::cout << "Thruster[" << i << "]-attitude: " << thrusters.back().positionAndAttitude[4] << "," << thrusters.back().positionAndAttitude[3] << std::endl;
		}
		//std::cout << "TOT" << Thr << std::endl << "FR---------" << std::endl;
		
		double contStates[] = {1,1,1,1,1,1};
		creator->GetDoubleParamArray("ControllableStates",controllableStates,6,contStates);
		for (size_t i =0;i<6;i++){
			if (controllableStates[i] == 0){
				for (size_t j =0;j<numberOfThrusters;j++){
				//	Thr(i,j) = 0;
				}
			}
		}
	//	std::cout << "TOT" << Thr << std::endl << "ETTER-------" << std::endl;
		Eigen::JacobiSVD<Eigen::MatrixXd> svdEigen(Thr, Eigen::ComputeFullU | Eigen::ComputeFullV);
		Eigen::MatrixXd sigmaInverse = Thr;
		sigmaInverse.setZero();
		for (int i = 0;(6<=numberOfThrusters)?i<6:i<numberOfThrusters;i++){
			if(svdEigen.singularValues()[i] > 1e-6){
				sigmaInverse(i,i) = 1.0/svdEigen.singularValues()[i];
			}
			else
				sigmaInverse(i,i) = 0;
		}
		TpseudoInverse = svdEigen.matrixV() * sigmaInverse * svdEigen.matrixU().transpose();

		for (size_t i = 0; i< numberOfThrusters;i++){
			K(i,i) = thrusters[i].RPMthrustCoeffs[0];//(thrusters.at(i).RPMthrustCoeffs[0]*thrusters.at(i).RPMlimits[1]*thrusters.at(i).RPMlimits[1] + thrusters.at(i).RPMthrustCoeffs[1]*thrusters.at(i).RPMlimits[1]);
			//K(i,i) = thrusters.at(i).RPMthrustCoeffs[0];
		}
	}
	//std::cout << "TOT" << Thr << std::endl << "Slutt---------" << std::endl;
	Kinv = K.inverse();
	creator->AddOutport("RPM",6,PORT_FUNCTION(ThrusterAllocation::RPM));											// Output port, Error
	creator->AddOutport("Thrust",6,PORT_FUNCTION(ThrusterAllocation::ThrustCommanded));								// Output port, Error
	creator->AddInport("Tau_desired",6,&Tau_desired);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		thrustFromRPM
//==================================================================================================
//
///	Calculates the thrust from a second order rpm-to-thrust equation based on the rpm commanded in the previous time step.
///		thrust = coeff1 * rpm^2 + coeff2 * rpm
///	
/// \param [in]		RPM					commanded RPM
///
/// \param [in]		thrusterNumber		index of the thruster in the thruster vector
///
/// \return								corresponding tau
///
/// 
////////////////////////////////////////////////////////////////////////////////////////////////////
const double* const ThrusterAllocation::thrustFromRPM(const double* const RPM, int thrusterNumber){
	/*
	if (linearRelation)
		thrusters.at(thrusterNumber).thrust = RPM[0] * thrusters.at(thrusterNumber).RPMthrustCoeffs[1];
	else
	*/	
	thrusters.at(thrusterNumber).thrust = sign(RPM[0])* thrusters.at(thrusterNumber).RPMthrustCoeffs[0] * RPM[0] * RPM[0]  + thrusters.at(thrusterNumber).RPMthrustCoeffs[1] *RPM[0];
	
	/*if (abs(RPM[0])> 1000){
		if (linearRelation)
			thrust = thrusters.at(thrusterNumber).RPMthrustCoeffs[1];
		else
			thrust = thrusters.at(thrusterNumber).RPMthrustCoeffs[0]*RPM[0] + thrusters.at(thrusterNumber).RPMthrustCoeffs[1];
	}
	else 
		thrust = thrusters.at(thrusterNumber).RPMthrustCoeffs[0] * 1000;*/
	return &thrusters.at(thrusterNumber).thrust;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		RPMfromU
//==================================================================================================
//
///					Calculates the desired rpm from the given u. 
///					Uses second order relation. n = sign(u) sqrt(abs(u))
///	
/// \param [in]		ui					calculated effort for a thruster
///
/// \param [in]		thrusterNumber		index of the thruster in the thruster vector
///
/// \return								corresponding RPM
///
/// 
////////////////////////////////////////////////////////////////////////////////////////////////////
const double* const ThrusterAllocation::RPMfromU(const double* const ui, int thrusterNumber){

	//if (linearRelation){
	//	//double iThrust = sign(double(ui[0])) *ui[0] * K(thrusterNumber,thrusterNumber);

	//	thrusters.at(thrusterNumber).rpm = 	ui[0];//iThrust / thrusters.at(thrusterNumber).RPMthrustCoeffs[1];

	//	
	//}
	//else {
		/*double iThrust = sign(double(ui[0])) *ui[0] * K(thrusterNumber,thrusterNumber);
		double A = thrusters.at(thrusterNumber).RPMthrustCoeffs[0];
		double B = thrusters.at(thrusterNumber).RPMthrustCoeffs[1];
		int iRPM = -B + sqrt(B*B + 4*A*iThrust) / (2*A);

		iRPM = iRPM*sign(double(ui[0]));
		thrusters.at(thrusterNumber).rpm = iRPM;*/

	thrusters.at(thrusterNumber).rpm = sign(ui[0]) * sqrt(abs(ui[0]));
	//}
	
	int maxRPM = thrusters.at(thrusterNumber).RPMlimits[1];
	int minRPM = thrusters.at(thrusterNumber).RPMlimits[0];
	int iRPM = thrusters.at(thrusterNumber).rpm;
	if (iRPM > maxRPM)
		thrusters.at(thrusterNumber).rpm = maxRPM;
	if(iRPM <minRPM)
		thrusters.at(thrusterNumber).rpm = minRPM;
	
	return &thrusters.at(thrusterNumber).rpm;

}

////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		OdeFnc
//==================================================================================================
//
///	Calculates the error in position and velocity, and assignes the position error to the state vector.
/// The error is defined as value - desired_value.
///	
/// \param [in]		T					current simulation time
///
/// \param [in]		X					current simulation state
///
/// \param [in]		XDot				pointer to the integrand
///
/// \param [in]		bIsMajorTimeStep	major time step flag
///
/// 
////////////////////////////////////////////////////////////////////////////////////////////////////
void ThrusterAllocation::OdeFcn(const double T, const double* const X, double* const XDot, const bool bIsMajorTimeStep){
	
	//if(bIsMajorTimeStep){
		const double* tauD = Tau_desired->GetPortValue(T,X);
		for (size_t i = 0;i<6;i++){
			tau(i) = tauD[i]*controllableStates[i];
		}
//		std::cout << tau<< "-----------------" << std::endl;
		u = Kinv/*.inverse()*/ * TpseudoInverse * tau;

		Eigen::VectorXd tempVector = Eigen::VectorXd::Zero(numberOfThrusters);
		for (size_t i = 0; i< numberOfThrusters;i++){
			RPMvalues[i] = RPMfromU(&u(i),i)[0];
			tempVector(i) = thrustFromRPM(&RPMvalues[i],i)[0];
		}

		tauCommand = Thr*tempVector;
	/*	std::cout << "TempVector:" << tempVector << std::endl << "----------------------------------" << std::endl;
		std::cout << "Kmatrix:" << K << std::endl  << "----------------------------------" << std::endl;
		std::cout << "Kinverse:" << Kinv << std::endl << "----------------------------------" << std::endl;
		std::cout << "Tmatrix:" << Thr << std::endl << "----------------------------------" << std::endl;*/
		
		
		//for (size_t i = 0;i<6;i++){
		//	tau(i) = Tau_desired->GetPortValue(T,X)[i];
		//}
		//for (size_t i = 0;i< numberOfThrusters;i++){
		//	K(i,i) = thrustFromRPM(&thrusters.at(i).previousRPM,i)[0];
		//}
		////std::cout << K << std::endl;

		//u = K.inverse() * TpseudoInverse * tau;

		//for (size_t i = 0; i< numberOfThrusters; i++){
		//	if (u(i) > 0 ){
		//		int maxRPM = thrusters.at(i).RPMlimits[1];
		//		if(u(i) > maxRPM){
		//			RPMvalues[i] = maxRPM;
		//		}
		//		else 
		//			RPMvalues[i] = u(i);
		//	}
		//	else if (u(i) <0){
		//		int minRPM = thrusters.at(i).RPMlimits[0];
		//		if(u(i) > minRPM){
		//			RPMvalues[i] = minRPM;
		//		}
		//		else 
		//			RPMvalues[i] = u(i);
		//	}
		//	else
		//		RPMvalues[i] = 0;
		//	u(i) = RPMvalues[i];
		//}
		//tauCommand = Thr*K*u;

		for (size_t i = 0; i< numberOfThrusters; i++){
			tauCommanded[i] = tauCommand(i);
		}
//	}

}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		RPM
//==================================================================================================
//
///	Returnes the calculated RPMs for the thrusters
///		
///	
/// \param [in]		T					dummy
///
/// \param [in]		X					dummy
///
/// \returns							RPM				
///
/// 
////////////////////////////////////////////////////////////////////////////////////////////////////
const double* const ThrusterAllocation::RPM(const double T, const double* const X){

	return RPMvalues;

}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		ThrustCommanded
//==================================================================================================
//
///	Calculates the thrust from a second order rpm-to-thrust equation based on the rpm commanded in the previous time step.
///		thrust = coeff1 * rpm^2 + coeff2 * rpm
///	
/// \param [in]		T					dummy
///
/// \param [in]		X					dummy
///
/// \return								tau commanded
///
/// 
////////////////////////////////////////////////////////////////////////////////////////////////////
const double* const ThrusterAllocation::ThrustCommanded(const double T, const double* const X){

	return tauCommanded;
}
