#include "Waypoint.h"
#include <assert.h>
#include <iostream>
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		Waypoint Constructor
//==================================================================================================
///		This constructor registers the position and radius of acceptence for a waypoint to be zero
///	
////////////////////////////////////////////////////////////////////////////////////////////////////
Waypoint::Waypoint():nPos(0),ePos(0),dPos(0),positionN(&nPos),positionE(&ePos),positionD(&dPos){
	for (size_t i =0;i<3;i++){
		position[i] = 0;
		attitude[i] = 0;
		velocity[i] = 0;
	}
}
//Waypoint::Waypoint(double* position, bool moving) : 
//	positionN(position), positionE(position +1), positionD(position+2), movingWP(moving){
//		for (size_t i =0;i<3;i++){
//		position[i] = 0;
//		attitude[i] = 0;
//		velocity[i] = 0;
//	}
//}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		Waypoint Constructor
//==================================================================================================
///		This constructor registers the position and radius of acceptence for a waypoint
///	
/// \param [in]		n					The N-position of the waypoint
///
/// \param [in]		e					The E-position of the waypoint
///
/// \param [in]		d					The D-position of the waypoint
///
/// \param [in]		radius				The radius of acceptance of the waypoint
///
////////////////////////////////////////////////////////////////////////////////////////////////////
Waypoint::Waypoint(double n, double e, double d, double radius) : 
	nPos(n), ePos(e), dPos(d),radiusOfAcceptence(radius)
	{
		positionN = &nPos;		positionE = &ePos;		positionD = &dPos;
		for (size_t i =0;i<3;i++){
		position[i] = 0;
		attitude[i] = 0;
		velocity[i] = 0;
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		Waypoint Constructor
//==================================================================================================
///		This constructor registers the position and radius of acceptence for a waypoint
///	
/// \param [in]		nPosition			The N-position of the waypoint
///
/// \param [in]		ePosition			The E-position of the waypoint
///
/// \param [in]		dPosition			The D-position of the waypoint
///
/// \param [in]		RadiusOfAcceptance	The radius of acceptance of the waypoint
///
////////////////////////////////////////////////////////////////////////////////////////////////////
Waypoint::Waypoint(double* const nPosition, double* const ePosition,double* const dPosition,double RadiusOfAcceptance) :
	radiusOfAcceptence(RadiusOfAcceptance), positionN(nPosition), positionE(ePosition), positionD(dPosition)
	{
		nPos = *nPosition;		ePos = *ePosition;		dPos = *dPosition;
		for (size_t i =0;i<3;i++){
		position[i] = 0;
		attitude[i] = 0;
		velocity[i] = 0;
	}
	}


////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		Attitude
//==================================================================================================
//
///	Returns the desired attitude when passing the waypoint.
///	
/// \return				desired attitude at the waypoint
////////////////////////////////////////////////////////////////////////////////////////////////////
double* Waypoint::Attitude(){ ///< Returns the desired heading when passing the waypoint
	return attitude;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		Speed
//==================================================================================================
//
///	Output port. Returns the desired velocity when passing way point
///
/// \return								Desired velocity when passing wapoint
///
////////////////////////////////////////////////////////////////////////////////////////////////////
double* Waypoint::Velocity(){ ///< Returns the desired speed when passing the waypoint
	return velocity;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		Position
//==================================================================================================
//
///	Output port. Returns current position.
///	
/// \return								position of the waypoint
///
////////////////////////////////////////////////////////////////////////////////////////////////////
double* Waypoint::Position(){ ///< Returns a pointer to the position-array of the waypoint
	position[0] = *positionN;
	position[1] = *positionE;
	position[2] = *positionD;

	return position;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		Radius
//==================================================================================================
//
/// Returns the radius of acceptence
///
/// \return								waypoint radius of acceptence
///
////////////////////////////////////////////////////////////////////////////////////////////////////
double Waypoint::Radius(){ 
	return radiusOfAcceptence;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		setAttitude
//==================================================================================================
//
/// Set the desired attitude at the waypoint
///
/// \param [in]	attitude		desired attitude at the way point
///
///
////////////////////////////////////////////////////////////////////////////////////////////////////
void Waypoint::setAttitude(double attitude[3]){
	attitude[0] = attitude[0];
	attitude[1] = attitude[1];
	attitude[2] = attitude[2];

}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		setRadiusOfAcceptance
//==================================================================================================
//
/// Sets the radius of acceptance for the way point
///
/// \param	[in] radius			radius of acceptance for the way point
///
///
////////////////////////////////////////////////////////////////////////////////////////////////////
void Waypoint::setRadiusOfAcceptance(double radius){
	radiusOfAcceptence = radius;

}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		setPosition
//==================================================================================================
//
/// Sets the position of the waypoint
///
///	\param [in]	n	north position
///	\param [in]	e	east position
///	\param [in]	d	down position
///
///
////////////////////////////////////////////////////////////////////////////////////////////////////
void Waypoint::setPosition(double* const n,double* const e,double* const d){
	positionN = n;
	positionE = e;
	positionD = d;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		setPosition
//==================================================================================================
//
/// Sets the postion of the waypoint
///
///	\param [in]	n	north position
///	\param [in]	e	east position
///	\param [in]	d	down position
///
///
////////////////////////////////////////////////////////////////////////////////////////////////////
void Waypoint::setPosition(double n,double e,double d){
	nPos = n;
	ePos = e;
	dPos = d;
}