////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							Waypoint
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Simple waypoint object
///
/// \author			Mats N{\aa}vik Hval
/// \par Revision history:
///					28.06.2011 MNH: Initial version.
///					06.06.2012 MNH: v2
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef WAYPOINT_H
#define WAYPOINT_H

class Waypoint
{
public:
	Waypoint();
//	Waypoint(double *position, bool moving);
	Waypoint(double* const NPosition, double* const EPosition, double* const DPosition,double radiusOfAcceptence);
	Waypoint(double NPosition, double EPosition, double DPosition, double radiusOfAcceptance); 
	void setAttitude(double attitude[3]);
	void setRadiusOfAcceptance(double radius);
	void setPosition(double* const n,double* const e,double* const d);
	void setPosition(double n,double e,double d);
	double* Position(); ///< Returns the position of the waypoint
	double* Attitude(); ///< Returns the desired heading when passing the waypoint
	double* Velocity(); ///< Returns the desired velocity when passing the waypoint
	double Radius(); ///< Returns the radius of acceptence. From within which radius of the current waypoint can the waypointchange occur. 
//	bool movingWP;					///< is the 

private: 
	double nPos;					///< Waypoint position N
	double ePos;					///< Waypoint position E
	double dPos;					///< Waypoint position D
	double *positionN;				///< Waypoint position as an array
	double *positionE;				///< Waypoint position as an array
	double *positionD;				///< Waypoint position as an array


	double position[3];					///< Position when passing the waypoint
	double attitude[3];					///< Desired attitude when passing the waypoint
	double velocity[3];					///< Desired velocity when passing the waypoint

	double radiusOfAcceptence;			///< The radius of acceptence. 
};
#endif

