#include "ConnectionPoint.h"
#include "Eigen\core"
#include "HelpFunctions.h"

ConnectionPoint::ConnectionPoint(const std::string& simObjectName, ISimObjectCreator* const creator) : SimObject(simObjectName)
{
	double zero[3] = {0,0,0};
	creator->GetDoubleParamArray("Offset",offset,3);

	creator->AddInport("ForcesNED",3,&forcesIn);
	creator->AddInport("OriginEta",6,&eta);
	//creator->AddInport("OriginVelocityNED",3,&velocityNED);

	creator->AddOutport("ForcesBODY",6,PORT_FUNCTION(ConnectionPoint::ForcesBODY));

	double zero6[] = {0,0,0,0,0,0};
	//indexDummy = creator->AddState("dummy",6,zero6);

}

const double* const ConnectionPoint::ForcesBODY(const double T, const double* const X){ 
	
	const double *forcesInArray = forcesIn->GetPortValue(T,X);
	//const double *eta_origin = positionNED->GetPortValue(T,X);
	const double *a_eta = eta->GetPortValue(T,X);

	Eigen::Matrix<double,3,1> forceNED;
	forceNED << forcesInArray[0],forcesInArray[1],forcesInArray[2];
	Eigen::Matrix<double,3,3> Rmat = RotationMatrix(Eigen::Matrix<double,3,1>(a_eta[3],a_eta[4],a_eta[5]));

	Eigen::Matrix<double,3,1> forceBODY = Rmat.transpose()*forceNED;


	forcesAndMoments[0] = forceBODY[0];
	forcesAndMoments[1] = forceBODY[1];
	forcesAndMoments[2] = forceBODY[2];
	forcesAndMoments[3] = -forceBODY[1]*offset[2] + forceBODY[2]*offset[1];
	forcesAndMoments[4] = -forceBODY[2]*offset[0] + forceBODY[0]*offset[2];
	forcesAndMoments[5] = -forceBODY[0]*offset[1] + forceBODY[1]*offset[0];
	
	return forcesAndMoments;
}

void ConnectionPoint::OdeFcn(const double T , const double* const X, double* const XDot,	const bool IsMajorTimeStep){
	//	
	//for (size_t i = 0; i <6 ; i++)
	//	XDot[indexDummy+i] = 0;
	
}

#ifdef FH_VISUALIZATION
	void ConnectionPoint::RenderInit(Ogre::Root* const ogreRoot, ISimObjectCreator* const creator){
	}

	void ConnectionPoint::RenderUpdate(const double T, const double* const X){
	}

#endif