////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							ConnectionPoint
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Point at the vessel where a forces are acting. Generates moments in addition to the force.
///
///
/// \author			Mats Nvik Hval
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef CONNECTIONPOINT_H
#define CONNECTIONPOINT_H

#include <vector>
#include <string>
#include "SimObject.h"

class ConnectionPoint : public SimObject
{
public:
	ConnectionPoint(const std::string& simObjectName, ISimObjectCreator* const creator);
	const double* const ForcesBODY(const double T, const double* const X); 
	void OdeFcn(const double T , const double* const X,	double* const XDot,	const bool IsMajorTimeStep); 

#ifdef FH_VISUALIZATION
	/**
	\brief Sets up visualization resources

	Creates a spherical mesh and registers at visualization system
		
	\param[in]	ogreRoot	-> Register visualization resources	
	\param[in]	creator		-> Retrieve parameters.
	*/
	void RenderInit(Ogre::Root* const ogreRoot, ISimObjectCreator* const creator);

	/**
	\brief Updates visualization resources to reflect current state
		
	Updates position of mesh

	\param[in]	T	-> Current simulation time
	\param[in]	X	-> Current simulation state
	*/
	void RenderUpdate(const double T, const double* const X);
#endif

protected:

private:

	ISignalPort* forcesIn;
	ISignalPort* eta;
	//ISignalPort* velocityNED;
	
	double offset[3];
	/*double eta_origin[6];
	double forcesInArray[3];
	*/double forcesAndMoments[6];

	int indexDummy;
	
};


#endif

