////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							ControlAlgorithm
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Superclass of control algorithms for the control system
///
///
/// \author			Mats Nvik Hval
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef CONTROLALGORITHM
#define CONTROLALGORITHM

#include "CSimObject.h"
#include <string>
#include <Eigen/Eigen>
using namespace Eigen;
#include <sfhGeneral.h>


class ControlAlgorithm : public CSimObject
{
public:
	ControlAlgorithm(const string& simObjectName, ISimObjectCreator* const creator); ///< reads parameters, registers states, input/output ports and shared resources
	virtual void OdeFcn(const double T, const double* const X, double* const XDot, const bool bIsMajorTimeStep); ///< returns object derivatives as a function of time, states and input ports

	
#ifdef FH_VISUALIZATION
	/// The rendering is not used.
	virtual void RenderInit(Ogre::Root* const ogreRoot, ISimObjectCreator* const creator){} ///< sets up visualization resources

	/// The rendering is not used.
	virtual void RenderUpdate(const double T, const double* const X){} ///< updates visualization resources to reflect current state.
#endif		

protected:
	ISignalPort* adStatePosition;	///< Current position
	ISignalPort* adStateTheta;		///< Current attitude in quaternions
	ISignalPort* adStateVelocity;	///< Current linear velocity
	ISignalPort* adStateRotation;	///< Current attitude in euler angles

	ISignalPort* adStatePosition_Desired;	///< Desired position
	ISignalPort* adStateTheta_Desired;		///< Desired attitude
	ISignalPort* adStateVelocity_Desired;	///< Desired linear velocity
	
	double* adStatePosition_error;		///< Position error, current - desired
	double* adStateTheta_error;			///< Attitude error, current - desired
	double* adStateVelocity_error;		///< Linear velocity error, current - desired
	
	int m_StatePositionErrorSum_Index;	///< The relative index of the StatePositionErrorSum state in the state array
	int m_StateThetaErrorSum_Index;		///< The relative index of the StateThetaErrorSum state in the state array
	int m_StateVelocityErrorSum_Index;	///< The relative index of the StateVelocityErrorSum state in the state array

};
#endif

