#include <Eigen/Eigen>
#include <math.h>
#include "FKForce.h"
#include <sfhGeneral.h>
#include "HelpFunctions.h"

using namespace std;
using namespace Eigen;
///TODO : READ ULEN,G,RHO

FKForce::FKForce(const string fileName, const string fileBaseLocation) : PotentialForce(fileName, fileBaseLocation){
	Read();
}

double* FKForce::GetForce(const double T, const double* const eta){
//	std::cout << "T= " << T << std::endl;

	///TODO: check the phase
	double xPos = eta[0];
	double yPos = eta[1];
	double zPos = -eta[2];
	double heading = eta[5];
	
	Matrix<double,6,1> tempForces;		tempForces.setZero();

	for (size_t i = 0; i< waveHeight.size();i++){
		double num1 = waveFrequency[i];
		double num2 = waveDirection[i];//+heading;
		double num3 = wavePhase[i];
		Matrix<double,6,1> phase = GetPhase(num1,num2);
		Matrix<double,6,1> positionTerm = Matrix<double,6,1>::Zero();
		for (size_t j =0;j<6;j++)
			positionTerm(j) = sin(waveFrequency[i]*T + num3 - waveNumber[i]*(sin(num2)*yPos+cos(num2)*xPos)/*sqrt(pow(xPos,2)+pow(xPos,2))*/ + (phase(j)*M_PI/180));
		double depthTerm = exp(-waveNumber[i]*(zPos));
		Matrix<double,6,1> amplitudeTerm = GetForceAmplitude(waveFrequency[i],waveDirection[i]-heading);

		for(size_t j = 0;j<6;j++)
			tempForces(j) += amplitudeTerm(j) *depthTerm *positionTerm(j);
	}
	for (size_t i = 0; i<6;i++)
		FroudeKrylovForce[i] = tempForces(i);
	
	return FroudeKrylovForce;	
}

Matrix<double,6,1> FKForce::GetForceAmplitude(double frequency, double waveHeading){

	waveHeading = RAD2PIPI(waveHeading);
	double lowerPeriodIndex = -1;
	double higherPeriodIndex = -1;
	double lowerHeadingIndex = -1;
	double higherHeadingIndex = -1;
	
	double wavePeriod; /// convert to wave period
	if (frequency < 1e-5){
		wavePeriod = potentialData_periods.back();
		higherPeriodIndex = potentialData_periods.size()-1;
	}
	else 
		wavePeriod = 2*M_PI / frequency;
	
	//find waveperiod above and below the input period
	if (wavePeriod <= potentialData_periods.at(0))
		lowerPeriodIndex = 0;
	
	if (lowerPeriodIndex < 0){
		for (size_t i = 0; i< potentialData_periods.size()-1;i++){
			if (potentialData_periods.at(i) <= wavePeriod && potentialData_periods.at(i+1) >= wavePeriod){
				lowerPeriodIndex = i;
				higherPeriodIndex = i+1;
			}
		}
	}
	double tempHeadingDEG = waveHeading * 180 / M_PI;
	bool negativeInputHeading = false;

	//find waveheading above and below the input heading
	if (tempHeadingDEG < 0){
		tempHeadingDEG = -tempHeadingDEG;
		negativeInputHeading = true;
	}
	for (size_t i = 0; i< potentialData_waveHeadings.size()-1;i++){
		if (potentialData_waveHeadings.at(i) <= tempHeadingDEG && potentialData_waveHeadings.at(i+1) >= tempHeadingDEG){
				lowerHeadingIndex = i;
				higherHeadingIndex = i+1;
			}
	}

	double scaleHeadingAbove = (tempHeadingDEG - potentialData_waveHeadings.at(lowerHeadingIndex)) / (potentialData_waveHeadings.at(higherHeadingIndex) - potentialData_waveHeadings.at(lowerHeadingIndex));//.at(higherHeadingIndex) - potentialData_waveHeadings.at(lowerHeadingIndex) / tempHeadingDEG;

	double scalePeriodAbove = (wavePeriod - potentialData_periods.at(higherPeriodIndex)) / (potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(lowerPeriodIndex));//potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(lowerPeriodIndex) / wavePeriod;

	Matrix<double,6,1> periodAbove = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> periodBelow = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> forceVector = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> negativeHeadingScale = Matrix<double,6,1>::Zero();
	negativeHeadingScale << 1, -1, 1, -1, 1, -1;
	
	for (size_t i = 0; i< 6 ;i++){
		periodAbove(i) = scaleHeadingAbove * FKdata.at(higherPeriodIndex).at(higherHeadingIndex).amplitude[i] + (1-scaleHeadingAbove) * FKdata.at(higherPeriodIndex).at(lowerHeadingIndex).amplitude[i];
		periodBelow(i) = scaleHeadingAbove * FKdata.at(lowerPeriodIndex).at(higherHeadingIndex).amplitude[i] + (1-scaleHeadingAbove) * FKdata.at(lowerPeriodIndex).at(lowerHeadingIndex).amplitude[i];
	}

	forceVector = scalePeriodAbove * periodAbove + (1-scalePeriodAbove) * periodBelow;
	
	if(negativeInputHeading){
		Matrix<double,6,1> temp = forceVector;
		for (size_t i=0;i<6;i++){
			forceVector(i) = temp(i) * negativeHeadingScale(i);
		}
	}
	return forceVector;
}


Matrix<double,6,1> FKForce::GetPhase(double frequency,double waveHeading){
	
	waveHeading  = RAD2PIPI(waveHeading);
	double lowerPeriodIndex = -1;
	double higherPeriodIndex = -1;
	double lowerHeadingIndex = -1;
	double higherHeadingIndex = -1;
	
	double wavePeriod; /// convert to wave period
	if (frequency < 1e-5){
		wavePeriod = potentialData_periods.back();
		higherPeriodIndex = potentialData_periods.size()-1;
	}
	else 
		wavePeriod = 2*M_PI / frequency;
	
	//find waveperiod above and below the input period
	if (wavePeriod <= potentialData_periods.at(0))
		lowerPeriodIndex = 0;
	
	if (lowerPeriodIndex < 0){
		for (size_t i = 0; i< potentialData_periods.size()-1;i++){
			if (potentialData_periods.at(i) <= wavePeriod && potentialData_periods.at(i+1) >= wavePeriod){
				lowerPeriodIndex = i;
				higherPeriodIndex = i+1;
			}
		}
	}
	double tempHeadingDEG = waveHeading * 180 / M_PI;
	bool negativeInputHeading = false;

	//find waveheading above and below the input heading
	if (tempHeadingDEG < 0){
		tempHeadingDEG = -tempHeadingDEG;
		negativeInputHeading = true;
	}
	for (size_t i = 0; i< potentialData_waveHeadings.size()-1;i++){
		if (potentialData_waveHeadings.at(i) <= tempHeadingDEG && potentialData_waveHeadings.at(i+1) >= tempHeadingDEG){
				lowerHeadingIndex = i;
				higherHeadingIndex = i+1;
			}
	}
	//fix scaling
	double scaleHeadingAbove = (tempHeadingDEG - potentialData_waveHeadings.at(lowerHeadingIndex)) / (potentialData_waveHeadings.at(higherHeadingIndex) - potentialData_waveHeadings.at(lowerHeadingIndex));//.at(higherHeadingIndex) - potentialData_waveHeadings.at(lowerHeadingIndex) / tempHeadingDEG;

	double scalePeriodAbove = (wavePeriod - potentialData_periods.at(higherPeriodIndex)) / (potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(lowerPeriodIndex));//potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(lowerPeriodIndex) / wavePeriod;

	Matrix<double,6,1> periodAbove = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> periodBelow = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> phaseVector = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> negativeHeadingScale = Matrix<double,6,1>::Zero();
	negativeHeadingScale << 1, -1, 1, -1, 1, -1;
	
	for (size_t i = 0; i< 6 ;i++){
		periodAbove(i) = scaleHeadingAbove * FKdata.at(higherPeriodIndex).at(higherHeadingIndex).phase[i] + (1-scaleHeadingAbove) * FKdata.at(higherPeriodIndex).at(lowerHeadingIndex).phase[i];
		periodBelow(i) = scaleHeadingAbove * FKdata.at(lowerPeriodIndex).at(higherHeadingIndex).phase[i] + (1-scaleHeadingAbove) * FKdata.at(lowerPeriodIndex).at(lowerHeadingIndex).phase[i];
	}

	phaseVector = scalePeriodAbove * periodAbove + (1-scalePeriodAbove) * periodBelow;
	
	/*if(negativeInputHeading){
		Matrix<double,6,1> temp = phaseVector;
		for (size_t i=0;i<6;i++){
			phaseVector(i) = temp(i) * negativeHeadingScale(i);
		}
	}*/
	/*Matrix<double,6,1> phaseVector;
	for (size_t i = 0; i<6 ;i++)
		phaseVector(i) = (FKdata.at(higherPeriodIndex).at(higherHeadingIndex).phase[i] + FKdata.at(higherPeriodIndex).at(lowerHeadingIndex).phase[i])/2 +  (FKdata.at(lowerPeriodIndex).at(higherHeadingIndex).phase[i] + FKdata.at(lowerPeriodIndex).at(lowerHeadingIndex).phase[i])/2;
	*/
	return phaseVector;
}


void FKForce::Read(){
	
	vector<double> tempWavePeriod;
	vector<double> tempWaveHeading;
		
	double GRAVITY,RHO,ULEN;
	
	ifstream file(potentialFileLocation + potentialFileName + ".out");
	string line;
	vector<string> tokens;
	while ( file.good() ){
		line.clear();
		getline(file,line);
		if(line.find("Gravity:",0) != -1){
			TokenizeString(line,tokens);
			GRAVITY = double(atof(tokens.at(1).c_str()));
			ULEN = double(atof(tokens.at(4).c_str()));
		}
		if(line.find("Water depth",0) != -1){
			TokenizeString(line,tokens);
			RHO = double(atof(tokens.at(5).c_str()));
		}
	}
	bool vectorNotBuilt = true;
	file.close();
	std::string fileName = potentialFileLocation + potentialFileName;

		file.open(fileName+".3");						///< import from wamit output file *.2  - forces from direct integration of hydrodynamic pressure
	//	string line;
		//vector<string> tokens;
		//vector<double> floatTokens; // vector containing the values in current line
	//	vector< vector< Matrix<double,6,4>>> RAOdata;
		//TODO : ce
		while ( file.good() ){
			line.clear();
			getline(file,line);
			TokenizeString(line,tokens);
			if (!tokens.empty()){
				//tempWaveHeading.clear();
				double periodValue = atof(tokens.at(0).c_str());
				double angleValue = atof(tokens.at(1).c_str());
				int dofValue = atoi(tokens.at(2).c_str());
				double amplitudeValue = atof(tokens.at(3).c_str());
				double phaseValue = atof(tokens.at(4).c_str());
				double realValue = atof(tokens.at(5).c_str());
				double imaginaryValue = atof(tokens.at(6).c_str());
			
				if (dofValue >3)
					amplitudeValue *= pow(ULEN,3)*RHO*GRAVITY;
				else 
					amplitudeValue *= pow(ULEN,2)*RHO*GRAVITY;
					
				if(tempWavePeriod.empty() && vectorNotBuilt){
					tempWavePeriod.push_back(periodValue);
					std::vector<FroudeKrylovStruct> newPeriod;
					FKdata.push_back(newPeriod);
				}
				else if (find(tempWavePeriod.begin(),tempWavePeriod.end(),periodValue) == tempWavePeriod.end()){//!(tempWavePeriod.back()*0.999 <= periodValue && periodValue <= tempWavePeriod.back()*1.001)){//
					tempWavePeriod.push_back(periodValue);
					std::vector<FroudeKrylovStruct> newPeriod;
					FKdata.push_back(newPeriod);
					tempWaveHeading.clear();
				}
				if(tempWaveHeading.empty()){
					FroudeKrylovStruct newHeading;
					FKdata.back().push_back(newHeading);
					tempWaveHeading.push_back(angleValue);
				}
				else if (find(tempWaveHeading.begin(),tempWaveHeading.end(),angleValue) == tempWaveHeading.end()) {//!(tempWaveHeading.back()*0.999 <= angleValue && angleValue <= tempWaveHeading.back()*1.001)){//
					//vector<Matrix<double,6,4>> raoMatrixVector;
					//raoMatrixVector.push_back(Matrix<double,6,4>::Zero());
					//RAOdata.push_back(raoMatrixVector);
					FroudeKrylovStruct newHeading;
					FKdata.back().push_back(newHeading);
					tempWaveHeading.push_back(angleValue);
				}
					if(dofValue > 3){
						FKdata.back().back().amplitude[dofValue-1] = amplitudeValue; //force amplitude for dof>3
						//FKdata.back().back().amplitude[dofValue-1] *= rho*g*pow(ulen,3);
						//RAOdata.back().back()(int(floatTokens[2])-1,0) = float(floatTokens[3]);	//force amplitude
					}
					else
						double a = FKdata.back().back().amplitude[0];
						FKdata.back().back().amplitude[dofValue-1] = amplitudeValue; //force amplitude for dof>3
					//	FKdata.back().back().amplitude[int(floatTokens[2])-1] *= rho*g*pow(ulen,2);}
					//	RAOdata.back().back()(int(floatTokens[2])-1,0) = float(floatTokens[3]);	//force amplitude

					if (dofValue == 3 ||	dofValue == 4 ||	dofValue == 5 )
						FKdata.back().back().phase[dofValue-1] = phaseValue+180; /// phase add pi to phase because of frame-transformation.
						//R	AOdata.back().back()(int(floatTokens[2])-1,1) = float(floatTokens[4])+float(180.0);				//force phase
					else
						FKdata.back().back().phase[dofValue-1] = phaseValue;
			
					if(dofValue>3){
						FKdata.back().back().re[dofValue-1] = realValue*RHO*pow(ULEN,3)*GRAVITY;	//force amplitude real part
						FKdata.back().back().im[dofValue-1] = imaginaryValue*RHO*pow(ULEN,3)*GRAVITY;	//force amplitude imaginary part
						//RAOdata.back().back()(int(floatTokens[2])-1,2) = float(floatTokens[5])*rho*pow(ulen,3)*g;	//force amplitude real part
						//RAOdata.back().back()(int(floatTokens[2])-1,3) = float(floatTokens[6])*rho*pow(ulen,3)*g;	//force amplitude real part
					}
					else {
						FKdata.back().back().re[dofValue-1] = realValue*RHO*pow(ULEN,2)*GRAVITY;	//force amplitude real part
						FKdata.back().back().im[dofValue-1] = imaginaryValue*RHO*pow(ULEN,2)*GRAVITY;	//force amplitude imaginary part
						}
					}
		}	

		file.close();
		potentialData_periods.clear();
		potentialData_waveHeadings.clear();

		for (size_t i = 0;i< tempWavePeriod.size();i++)
			potentialData_periods.push_back(tempWavePeriod.at(i));
	
		for (size_t i = 0;i< tempWaveHeading.size();i++)
			potentialData_waveHeadings.push_back(tempWaveHeading.at(i));
	}
	