////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							FKForce
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Subclass of PotentialForce representing the potential Froude Krylov forces and moments
///
///
/// \author			Mats Nvik Hval
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef FKFORCE_H
#define FKFORCE_H

#include <vector>
#include <string>
#include <fstream>
#include "PotentialForce.h"



#ifndef FROUDEKRYLOVSTRUCT
#define FROUDEKRYLOVSTRUCT
struct FroudeKrylovStruct{
	double amplitude[6];
	double phase[6];
	double im[6];
	double re[6];
};
#endif
class FKForce : public PotentialForce
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	FKForce(const std::string fileName, const std::string fileBaseLocation);
	double* GetForce(const double T, const double* const X);
	void Read();
protected:

private:
	Eigen::Matrix<double,6,1> GetForceAmplitude(double frequency, double waveHeading);
	Eigen::Matrix<double,6,1> GetPhase(double frequency,double waveHeading);	
	std::vector<std::vector<FroudeKrylovStruct>> FKdata;
	std::ifstream file;
	double forceArray[6];
	double phaseArray[6];

	int previousPeriodIndex;
	int previousHeadingIndex;

	
};
#endif

