////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							FluidMemoryForceElement
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Class for one element of the fluid memory effect as in MSS FDI Toolbox by Perez and Fossen. 
///					<www.marinecontrol.org>
///
///
/// \author			Mats Nvik Hval
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef FLUIDMEMORYELEMENT_H
#define FLUIDMEMORYELEMENT_H

#include "SimObject.h"
#include "Eigen/Core"

class FluidMemoryElement : public SimObject
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	~FluidMemoryElement();
	FluidMemoryElement(const string& simObjectName, ISimObjectCreator* const creator); ///< Reads parameters, registers states, input/output ports and shared resources
	void OdeFcn(const double T, const double* const X, double* const XDot, const bool bIsMajorTimeStep); ///< returns object derivatives as a function of time, states and input ports
	//const double* const Forces(const double T, const double* const X);///< Returns the angle of the rudder 
	const double* const GetValue(const double T, const double* const X);

#ifdef FH_VISUALIZATION
	void			RenderInit(Ogre::Root* const ogreRoot, ISimObjectCreator* const creator); ///< sets up visualization resources
	void			RenderUpdate(const double T, const double* const X); ///< updates visualization resources to reflect current state
#endif
private:
	ISignalPort* velocity;
	ISignalPort* position;

	int orderOfSystem;
	int dofs[2];
	vector<int> indexState;
	Eigen::MatrixXd A;
	Eigen::MatrixXd B;
	Eigen::MatrixXd C;
	Eigen::MatrixXd D;

	double *Ar;
	double *Br;
	double *Cr;
	double Dr;
	double outValue;
	

};

#endif
