#ifndef FroudeKrylov_h__
#define FroudeKrylov_h__

using std::string;
using std::vector;
using std::map;
using std::complex;

using Eigen::Matrix;

class FroudeKrylovForce
{
public:

	virtual void Read( std::string fname )=0;
	
	virtual std::complex<double> GetRAOValue(unsigned int DOF,  double ShipDirection, double ShipSpeed, double WaveFrequency ) = 0;
	virtual void GetRAOValues( unsigned int DOF, double ShipDirection, double ShipSpeed, Eigen::Matrix< std::complex<double>,Eigen::Dynamic,1 > &outRAO)=0;

	virtual void GetRAOValue2(unsigned int DOF,  double ShipDirection, double ShipSpeed, double WaveFrequency, double &Magnitude, double &Phase ) = 0;
	virtual void GetRAOValues2( unsigned int DOF, double ShipDirection, double ShipSpeed, Eigen::Matrix< double,Eigen::Dynamic,1 > &outRAOmag,Eigen::Matrix< double,Eigen::Dynamic,1 > &outRAOphase)=0;


protected:
	int m_NumDirections;	///< Number of wave directions relative to ship in dataset
	int m_NumSpeed;			///< Number of vessel speeds in dataset
	int m_NumFreq;			///< Number of frequencies in dataset
	int m_NumDOF;			///< Number of degrees-of-freedom in dataset

	std::vector<double> Direction;	///< Directions
	std::vector<double> Speed;		///< Speeds
	std::vector<double> Freq;		///< Frequencies
	Eigen::Matrix< std::complex<double>, Eigen::Dynamic,1 > ***RAO;	///< map which associates a direction/speed pair with an RAO on complex form
	Eigen::Matrix< double, Eigen::Dynamic,1 > ***RAOmagnitude;
	Eigen::Matrix< double, Eigen::Dynamic,1 > ***RAOphase;
};
#endif // FroudeKrylov_h__