#include "HelpFunctions.h"

using namespace std;
using namespace Eigen;

#define PI 3.14159265


void TokenizeString(const string &line,vector<string> &tokens)
{
	string line2 = line;
	tokens.clear();
	size_t pos = line2.find_first_of('\t');

	while (string::npos != pos){
		line2[pos] = ' ';
		pos = line2.find_first_of('\t');
	}
	size_t lastPos = line2.find_first_not_of(" ",0);
	
	pos = line2.find_first_of(" ",lastPos);

	while (string::npos != pos || string::npos !=lastPos){
		tokens.push_back(line2.substr(lastPos,pos-lastPos));
		lastPos = line2.find_first_not_of(" ",pos);
		pos = line2.find_first_of(" ",lastPos);
	}
}
void DiscreteToContinousScaling(const double minX, const double maxX, const double currentValue, double *alpha1, double *alpha2)
{
	/// Based on first-order-hold 

	*alpha1 = (currentValue - minX) / (maxX - minX);
	*alpha2 = 1- *alpha1;
}



void FindIndices(unsigned int *indexBelow, unsigned int *indexAbove, double value, std::vector<double> vectorOfValues,const unsigned int indexOfLastBelow, const unsigned int indexOfLastAbove)
{
	unsigned int upperIndex = 0;
	unsigned int lowerIndex = 0;
	unsigned int oneBelowPreviousValue = (indexOfLastBelow-1 >1)?indexOfLastBelow-1:1;
	unsigned int twoBelowPreviousValue = (indexOfLastBelow-2 >1)?indexOfLastBelow-2:1;
	unsigned int oneAbovePreviousValue = (indexOfLastAbove+1 >1)?indexOfLastAbove+1:1;
	unsigned int twoAbovePreviousValue = (indexOfLastAbove+2 >1)?indexOfLastAbove+2:1;
	
	if (value < vectorOfValues[indexOfLastAbove] && value >= vectorOfValues[indexOfLastBelow])
	{
		upperIndex = indexOfLastAbove;
		lowerIndex = indexOfLastBelow;
	}
	else if ( value < vectorOfValues[oneAbovePreviousValue] && value >= vectorOfValues[indexOfLastAbove])
		{
			upperIndex = oneAbovePreviousValue;
			lowerIndex = indexOfLastAbove;
		}
	else if ( value < vectorOfValues[twoAbovePreviousValue] && value >= vectorOfValues[oneAbovePreviousValue])
		{
			upperIndex = twoAbovePreviousValue;
			lowerIndex = oneAbovePreviousValue;
		}
	else if ( value < vectorOfValues[indexOfLastBelow] && value >= vectorOfValues[oneBelowPreviousValue])
		{
			upperIndex = indexOfLastBelow;
			lowerIndex = oneBelowPreviousValue;
		}
	else if ( value < vectorOfValues[oneBelowPreviousValue] && value >= vectorOfValues[twoBelowPreviousValue])
		{
			upperIndex = oneBelowPreviousValue;
			lowerIndex = twoBelowPreviousValue;
		}
	else 
	{
	
		//unsigned int upperIndex = vectorOfValues.size()-1;
		//unsigned int lowerIndex = 1;

		if (value == vectorOfValues.back()) //(length(wavePeriod)))
		{
			upperIndex = vectorOfValues.size()-1;
			lowerIndex = vectorOfValues.size()-1;
		}
		else if (value == vectorOfValues[1])
		{
			upperIndex = 1;
			lowerIndex = 1;
		}
		else if (value < vectorOfValues.back() && value>0)
		{
			while (upperIndex-lowerIndex > 1 && value >= vectorOfValues[lowerIndex] && value <= vectorOfValues[upperIndex])
			{
				unsigned int splitterIndex = min(lowerIndex,upperIndex) + (upperIndex - lowerIndex) /2;
				if ( value > vectorOfValues[splitterIndex])
					lowerIndex = splitterIndex;
				if ( value < vectorOfValues[splitterIndex])
					upperIndex = splitterIndex;
				if (upperIndex - lowerIndex == 2 && value == vectorOfValues[lowerIndex+1])
				{
					upperIndex = lowerIndex+1;
					lowerIndex = lowerIndex+1;
				}
			}
		}
		else
		{
			lowerIndex = vectorOfValues.size()-1;
			upperIndex = 0;
		}
	
		* indexAbove = upperIndex;
		* indexBelow = lowerIndex;
	}
}

void RotationMatrix(Eigen::Matrix3d *rotationMatrix, double phi, double theta, double psi){
	
	double cPhi = cos(phi);
	double sPhi = sin(phi);
	
	double cTheta = cos(theta);
	double sTheta = sin(theta);
	
	double cPsi = cos(psi);
	double sPsi = sin(psi);

	rotationMatrix->topRows(1) << cPsi*cTheta, -sPsi*cPhi + cPsi*sTheta*sPhi, sPsi*sPhi + cPsi*cPhi*sTheta;
	rotationMatrix->middleRows<1>(1) << sPsi*cTheta, cPsi*cPhi + sPhi*sTheta*sPsi, -cPsi*sPhi + sTheta*sPsi*cPhi;
	rotationMatrix->bottomRows(1) << -sTheta, cTheta*sPhi, cTheta*cPhi;
}

Eigen::Matrix3d RotationMatrix(Eigen::Vector3d vector){
	Matrix3d rotationMatrix;
	
	double cPhi = cos(vector(0));
	double sPhi = sin(vector(0));
	
	double cTheta = cos(vector(1));
	double sTheta = sin(vector(1));
	
	double cPsi = cos(vector(2));
	double sPsi = sin(vector(2));

	rotationMatrix.topRows(1) << cPsi*cTheta, -sPsi*cPhi + cPsi*sTheta*sPhi, sPsi*sPhi + cPsi*cPhi*sTheta;
	rotationMatrix.middleRows<1>(1) << sPsi*cTheta, cPsi*cPhi + sPhi*sTheta*sPsi, -cPsi*sPhi + sTheta*sPsi*cPhi;
	rotationMatrix.bottomRows(1) << -sTheta, cTheta*sPhi, cTheta*cPhi;
	
	return rotationMatrix;
}

void Smtrx(Matrix3d *skewSymmetricMatrix,double value1, double value2, double value3)
{
	*skewSymmetricMatrix << 0, -value3, value2, value3, 0, -value1, -value2, value1,0;
}

Eigen::Matrix3d Smtrx(Eigen::Vector3d vector){

	Eigen::Matrix3d skewSymmetricMatrix;
	skewSymmetricMatrix << 0, -vector(2), vector(1), vector(2), 0, -vector(0), -vector(1), vector(0),0;

	return skewSymmetricMatrix;
}

Eigen::Matrix3d TransformationMatrix(Eigen::Vector3d vector){
	Eigen::Matrix3d transformationMatrix;

	transformationMatrix.row(0) << 1,sin(vector[0])*tan(vector[1]),cos(vector[0])*tan(vector[1]);
	transformationMatrix.row(1) << 0, cos(vector[0]), -sin(vector[0]);
	transformationMatrix.row(2) << 0, sin(vector[0])/cos(vector[1]), cos(vector[0])/cos(vector[1]);
	return transformationMatrix;
}

Eigen::Matrix<double,6,6> Jmatrix(const double * const eta){

	Matrix3d R = RotationMatrix(Eigen::Vector3d(eta[3],eta[4],eta[5]));
	Matrix3d T = TransformationMatrix(Eigen::Vector3d(eta[3],eta[4],eta[5]));

	Matrix<double,6,6> J = Matrix<double,6,6>::Zero();

	J.topLeftCorner(3,3) = R;
	J.bottomRightCorner(3,3) = T;

	return J;
}
//const double* const Omega2Quaternion(double phi, double theta, double psi){
//
//	Matrix<float,3,3> rotMat = RotationMatrix(Eigen::Vector3f(float(phi),(float)theta,float(psi)));
//	double quat[4];
//
//	double trace = rotMat.diagonal().sum();
//
//}

double Modulus(double a, double b)
 {
 int result = static_cast<int>( a / b );
 return a - static_cast<double>( result ) * b;
 }

int sign(double value){

	int sign = 0;

	if (value > 0)
		sign =1;
	if (value < 0)
		sign = -1;

	return sign;
}
double RAD2PIPI(double angleIn){
	double temp = fmod(angleIn,2*sfh::pi);
	if (abs(temp) > sfh::pi){
		//double angle = Reminder(angleIn,2*M_PI);
		int signAngle = sfh::Sign(temp);
		temp = -signAngle*(2*sfh::pi - signAngle*temp);
	}
	/*double r = Modulus(angleIn+sign(angleIn)*M_PI,2*M_PI);
	double s = sign(sign(angleIn)+2*(sign(abs(Modulus(angleIn+M_PI,2*M_PI)/(2*M_PI)))-1));
	double result = r-s*M_PI;*/
	return temp;
}