#include <Eigen/StdVector>
#include <math.h>
#include "MeanDriftForce.h"
#include <sfhGeneral.h>
#include "HelpFunctions.h"

using namespace std;
using namespace Eigen;
///TODO : READ ULEN,G,RHO

MeanDriftForce::MeanDriftForce(const std::string fileName, const std::string fileBaseLocation) : PotentialForce(fileName,fileBaseLocation){
	Read();
}

double* MeanDriftForce::GetForce(const double T, const double* const eta){
		///TODO: check the phase
	double xPos = eta[0];
	double yPos = eta[1];
	double zPos = -eta[2];
	double heading = eta[5];
	
	Matrix<double,6,1> tempForces;		tempForces.setZero();

	for (size_t i = 0; i< waveHeight.size();i++){
		Matrix<double,6,1> phase = GetPhase(waveFrequency[i],waveDirection[i]+heading);
		Matrix<double,6,1> positionTerm;// = Eigen::Matrix<double,6,1>::Zero();
		for (size_t j =0;j<6;j++)
			positionTerm(j) = sin(waveFrequency[i]*T + waveNumber[i]*sqrt(pow(xPos,2)+pow(xPos,2)) + phase(j)*M_PI/180);
		double depthTerm =  exp(-waveNumber[i]*(zPos));
		Matrix<double,6,1> amplitudeTerm = GetForceAmplitude(waveFrequency[i],waveDirection[i]+heading);
		for(size_t j = 0;j<6;j++)
			tempForces(j) += amplitudeTerm(j) *depthTerm *positionTerm(j);
		}
	for (size_t i = 0; i<6;i++)
		MeanWaveDriftForce[i] = tempForces(i);
	
	return MeanWaveDriftForce;	
}


Matrix<double,6,1> MeanDriftForce::GetForceAmplitude(double frequency,double waveHeading){

	double lowerPeriodIndex = -1;
	double higherPeriodIndex = -1;
	double lowerHeadingIndex = -1;
	double higherHeadingIndex = -1;
	
	double wavePeriod; /// convert to wave period
	if (frequency < 1e-5){
		wavePeriod = potentialData_periods.back();
		higherPeriodIndex = potentialData_periods.size()-1;
	}
	else 
		wavePeriod = 2*M_PI / frequency;
	
	//find waveperiod above and below the input period
	if (wavePeriod <= potentialData_periods.at(0))
		lowerPeriodIndex = 0;
	
	if (lowerPeriodIndex < 0){
		for (size_t i = 0; i< potentialData_periods.size()-1;i++){
			if (potentialData_periods.at(i) <= wavePeriod && potentialData_periods.at(i+1) >= wavePeriod){
				lowerPeriodIndex = i;
				higherPeriodIndex = i+1;
			}
		}
	}
	double tempHeadingDEG = waveHeading * 180 / M_PI;
	bool negativeInputHeading = false;

	//find waveheading above and below the input heading
	if (tempHeadingDEG < 0){
		tempHeadingDEG = tempHeadingDEG + 180;
		negativeInputHeading = true;
	}
	for (size_t i = 0; i< potentialData_waveHeadings.size()-1;i++){
		if (potentialData_waveHeadings.at(i) <= tempHeadingDEG && potentialData_waveHeadings.at(i+1) >= tempHeadingDEG){
				lowerHeadingIndex = i;
				higherHeadingIndex = i+1;
			}
	}

	double scaleHeadingAbove = (tempHeadingDEG - potentialData_waveHeadings.at(lowerHeadingIndex)) - (potentialData_waveHeadings.at(higherHeadingIndex) - potentialData_waveHeadings.at(higherHeadingIndex));//.at(higherHeadingIndex) - potentialData_waveHeadings.at(lowerHeadingIndex) / tempHeadingDEG;

	double scalePeriodAbove = (wavePeriod - potentialData_periods.at(higherPeriodIndex)) - (potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(higherPeriodIndex));//potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(lowerPeriodIndex) / wavePeriod;

	Matrix<double,6,1> periodAbove = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> periodBelow = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> forceVector = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> negativeHeadingScale = Matrix<double,6,1>::Zero();
	negativeHeadingScale << 1, -1, 1, -1, 1, -1;
	
	for (size_t i = 0; i< 6 ;i++){
		periodAbove(i) = scaleHeadingAbove * MeanDriftdata.at(higherPeriodIndex).at(higherHeadingIndex).amplitude[i] + (1-scaleHeadingAbove) * MeanDriftdata.at(higherPeriodIndex).at(lowerHeadingIndex).amplitude[i];
		periodBelow(i) = scaleHeadingAbove * MeanDriftdata.at(lowerPeriodIndex).at(higherHeadingIndex).amplitude[i] + (1-scaleHeadingAbove) * MeanDriftdata.at(lowerPeriodIndex).at(lowerHeadingIndex).amplitude[i];
	}

	forceVector = scalePeriodAbove * periodAbove + (1-scalePeriodAbove) * periodBelow;
	
	if(negativeInputHeading)
		for(size_t i =0;i<6;i++)
			forceVector(i) *= negativeHeadingScale(i);
	
	return forceVector;

}

Matrix<double,6,1> MeanDriftForce::GetPhase(double frequency,double waveHeading){

	double lowerPeriodIndex = -1;
	double higherPeriodIndex = -1;
	double lowerHeadingIndex = -1;
	double higherHeadingIndex = -1;
	
	double wavePeriod; /// convert to wave period
	if (frequency < 1e-5){
		wavePeriod = potentialData_periods.back();
		higherPeriodIndex = potentialData_periods.size()-1;
	}
	else 
		wavePeriod = 2*M_PI / frequency;
	
	//find waveperiod above and below the input period
	if (wavePeriod <= potentialData_periods.at(0))
		lowerPeriodIndex = 0;
	
	if (lowerPeriodIndex < 0){
		for (size_t i = 0; i< potentialData_periods.size()-1;i++){
			if (potentialData_periods.at(i) <= wavePeriod && potentialData_periods.at(i+1) >= wavePeriod){
				lowerPeriodIndex = i;
				higherPeriodIndex = i+1;
			}
		}
	}
	double tempHeadingDEG = waveHeading * 180 / M_PI;
	bool negativeInputHeading = false;

	//find waveheading above and below the input heading
	if (tempHeadingDEG < 0){
		tempHeadingDEG = -tempHeadingDEG;// + 180;
		negativeInputHeading = true;
	}
	for (size_t i = 0; i< potentialData_waveHeadings.size()-1;i++){
		if (potentialData_waveHeadings.at(i) <= tempHeadingDEG && potentialData_waveHeadings.at(i+1) >= tempHeadingDEG){
				lowerHeadingIndex = i;
				higherHeadingIndex = i+1;
			}
	}
	double scaleHeadingAbove = (tempHeadingDEG - potentialData_waveHeadings.at(lowerHeadingIndex)) - (potentialData_waveHeadings.at(higherHeadingIndex) - potentialData_waveHeadings.at(higherHeadingIndex));//.at(higherHeadingIndex) - potentialData_waveHeadings.at(lowerHeadingIndex) / tempHeadingDEG;
	
	double scalePeriodAbove = (wavePeriod - potentialData_periods.at(higherPeriodIndex)) - (potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(higherPeriodIndex));//potentialData_periods.at(higherPeriodIndex) - potentialData_periods.at(lowerPeriodIndex) / wavePeriod;

	Matrix<double,6,1> periodAbove = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> periodBelow = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> phaseVector = Matrix<double,6,1>::Zero();
	Matrix<double,6,1> negativeHeadingScale = Matrix<double,6,1>::Zero();
	negativeHeadingScale << 1, -1, 1, -1, 1, -1;
	
	for (size_t i = 0; i< 6 ;i++){
		periodAbove(i) = scaleHeadingAbove * MeanDriftdata.at(higherPeriodIndex).at(higherHeadingIndex).phase[i] + (1-scaleHeadingAbove) * MeanDriftdata.at(higherPeriodIndex).at(lowerHeadingIndex).phase[i];
		periodBelow(i) = scaleHeadingAbove * MeanDriftdata.at(lowerPeriodIndex).at(higherHeadingIndex).phase[i] + (1-scaleHeadingAbove) * MeanDriftdata.at(lowerPeriodIndex).at(lowerHeadingIndex).phase[i];
	}

	phaseVector = scalePeriodAbove * periodAbove + (1-scalePeriodAbove) * periodBelow;

	/*Matrix<double,6,1> phaseVector;			phaseVector.setZero();
	for (size_t i = 0; i<6 ;i++)
		phaseVector(i) = (MeanDriftdata.at(higherPeriodIndex).at(higherHeadingIndex).phase[i] + MeanDriftdata.at(higherPeriodIndex).at(lowerHeadingIndex).phase[i])/2 +  (MeanDriftdata.at(lowerPeriodIndex).at(higherHeadingIndex).phase[i] + MeanDriftdata.at(lowerPeriodIndex).at(lowerHeadingIndex).phase[i])/2;
	*/
	return phaseVector;
}
void MeanDriftForce::Read(){
	
	vector<double> tempWavePeriod;
	vector<double> tempWaveHeading;
		
	double GRAVITY,RHO,ULEN;
	
	ifstream file(potentialFileName + ".out");
	string line;
	vector<string> tokens;
	while ( file.good() ){
		line.clear();
		getline(file,line);
		if(line.find("Gravity:",0) != -1){
			TokenizeString(line,tokens);
			GRAVITY = double(atof(tokens.at(1).c_str()));
			ULEN = double(atof(tokens.at(4).c_str()));
		}
		if(line.find("Water depth",0) != -1){
			TokenizeString(line,tokens);
			RHO = double(atof(tokens.at(5).c_str()));
		}
	}

	file.close();

	file.open(potentialFileName+".9");						///< import from wamit output file *.3  - forces from direct integration of hydrodynamic pressure
//	vector<double> floatTokens; // vector containing the values in current line
//	vector< vector< Matrix<double,6,4>>> RAOdata;
	
	while ( file.good() ){
		line.clear();
		getline(file,line);
		TokenizeString(line,tokens);
		if (!tokens.empty() &&  atoi(tokens[3].c_str()) >0){
			double periodValue = atof(tokens.at(0).c_str());
			double angleValue = atof(tokens.at(1).c_str());
			int dofValue = atoi(tokens.at(3).c_str());
			double amplitudeValue = atof(tokens.at(4).c_str());
			double phaseValue = atof(tokens.at(5).c_str());
			double realValue = atof(tokens.at(6).c_str());
			double imaginaryValue = atof(tokens.at(7).c_str());
			
			if (dofValue >3)
				amplitudeValue *= pow(ULEN,3)*RHO*GRAVITY;
			else 
				amplitudeValue *= pow(ULEN,2)*RHO*GRAVITY;
					
			if(tempWavePeriod.empty()){
				tempWavePeriod.push_back(periodValue);
				std::vector<MeanDriftStruct> newPeriod;
				MeanDriftdata.push_back(newPeriod);
			}
			else if (find(tempWavePeriod.begin(),tempWavePeriod.end(),periodValue) == tempWavePeriod.end()){//!(tempWavePeriod.back()*0.999 <= periodValue && periodValue <= tempWavePeriod.back()*1.001)){//
				tempWavePeriod.push_back(periodValue);
				std::vector<MeanDriftStruct> newPeriod;
				MeanDriftdata.push_back(newPeriod);
				tempWaveHeading.clear();
			}
			if(tempWaveHeading.empty()){
				MeanDriftStruct newHeading;
				MeanDriftdata.back().push_back(newHeading);
				tempWaveHeading.push_back(angleValue);
			}
			else if (find(tempWaveHeading.begin(),tempWaveHeading.end(),angleValue) == tempWaveHeading.end()) {//!(tempWaveHeading.back()*0.999 <= angleValue && angleValue <= tempWaveHeading.back()*1.001)){//
				//vector<Matrix<double,6,4>> raoMatrixVector;
				//raoMatrixVector.push_back(Matrix<double,6,4>::Zero());
				//RAOdata.push_back(raoMatrixVector);
				MeanDriftStruct newHeading;
				MeanDriftdata.back().push_back(newHeading);
				tempWaveHeading.push_back(angleValue);
			}
			if(dofValue > 3){
				MeanDriftdata.back().back().amplitude[dofValue-1] = amplitudeValue; //force amplitude for dof>3
				//FKdata.back().back().amplitude[dofValue-1] *= rho*g*pow(ulen,3);
				//RAOdata.back().back()(int(floatTokens[2])-1,0) = float(floatTokens[3]);	//force amplitude
			}
			else
				MeanDriftdata.back().back().amplitude[dofValue-1] = amplitudeValue; //force amplitude for dof>3
			//	FKdata.back().back().amplitude[int(floatTokens[2])-1] *= rho*g*pow(ulen,2);}
			//	RAOdata.back().back()(int(floatTokens[2])-1,0) = float(floatTokens[3]);	//force amplitude
						
			if (dofValue == 3 ||	dofValue == 4 ||	dofValue == 5 )
				MeanDriftdata.back().back().phase[dofValue-1] = phaseValue+180; /// phase add pi to phase because of frame-transformation.
				//R	AOdata.back().back()(int(floatTokens[2])-1,1) = float(floatTokens[4])+float(180.0);				//force phase
			else
				MeanDriftdata.back().back().phase[dofValue-1] = phaseValue;
			
			if(dofValue>3){
				MeanDriftdata.back().back().re[dofValue-1] = realValue*RHO*pow(ULEN,3)*GRAVITY;	//force amplitude real part
				MeanDriftdata.back().back().im[dofValue-1] = imaginaryValue*RHO*pow(ULEN,3)*GRAVITY;	//force amplitude imaginary part
				//RAOdata.back().back()(int(floatTokens[2])-1,2) = float(floatTokens[5])*rho*pow(ulen,3)*g;	//force amplitude real part
				//RAOdata.back().back()(int(floatTokens[2])-1,3) = float(floatTokens[6])*rho*pow(ulen,3)*g;	//force amplitude real part
			}
			else {
				MeanDriftdata.back().back().re[dofValue-1] = realValue*RHO*pow(ULEN,2)*GRAVITY;	//force amplitude real part
				MeanDriftdata.back().back().im[dofValue-1] = imaginaryValue*RHO*pow(ULEN,2)*GRAVITY;	//force amplitude imaginary part
			}
		}
	}
		

	file.close();
	potentialData_periods.clear();
	potentialData_waveHeadings.clear();
	for (size_t i = 0;i< tempWavePeriod.size();i++)
		potentialData_waveHeadings.push_back(tempWavePeriod.at(i));
		
	for (size_t i = 0;i< tempWaveHeading.size();i++)
		potentialData_waveHeadings.push_back(tempWaveHeading.at(i));
	
	
}