////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							MeanDriftForce
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Subclass of PotentialForce representing the potential mean drift forces and moments
///
///
/// \author			Mats Nvik Hval
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef MEANDRIFTFORCE_H
#define MEANDRIFTFORCE_H

#include <vector>
#include <string>
#include <fstream>
#include "PotentialForce.h"


#ifndef MEANDRIFTSTRUCT
#define MEANDRIFTSTRUCT
struct MeanDriftStruct{
	double amplitude[6];
	double phase[6];
	double im[6];
	double re[6];
};
#endif
class MeanDriftForce : public PotentialForce
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	MeanDriftForce(const std::string fileName, const std::string fileBaseLocation);
	double* GetForce(const double T,const double* const eta);
	void Read();
protected:

private:
	
	Eigen::Matrix<double,6,1> GetPhase(double frequency,double waveHeading);	
	Eigen::Matrix<double,6,1> GetForceAmplitude(double frequency, double waveHeading);
	std::vector<std::vector<MeanDriftStruct>> MeanDriftdata;
	std::ifstream file;


	
};
#endif

