////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							PotentialForce
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Superclass of forces caused by the fluid velocity potential
///
///
/// \author			Mats Nvik Hval
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef POTENTIALFORCE_H
#define POTENTIALFORCE_H

#include <string>
#include <Eigen/Eigen>
#include <math.h>
#include <sfhGeneral.h>
#include <DeepSeaGravityWaves.h>
#include "Eigen/StdVector"

class PotentialForce{

public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	PotentialForce(const std::string fileName, const string fileBaseLocation);
	virtual double* GetForce(const double T, const double* const X) = 0;//double frequency,double waveHeading) = 0;
//	virtual double* GetPhase(double frequency,double waveHeading) = 0;
	virtual void Read() = 0;
	void setEnvironment(DeepSeaGravityWaves* environment);
protected:
	/*double rho;												///< Fluid density
	double g;												///< Gravitational constant
	double ulen;											///< Characteristic length of vessel*/
	double FroudeKrylovForce[6];
	double MeanWaveDriftForce[6];

	std::string potentialFileName;							///< Filename of the potential files
	std::string potentialFileLocation;						///< Location of the potential files

	DeepSeaGravityWaves* environment;
	std::vector<double> waveHeight;
	std::vector<double> waveFrequency;
	std::vector<double> waveNumber;
	std::vector<double> wavePhase;
	std::vector<double> waveDirection;

	double seaDepth;
	
	std::vector<double> potentialData_waveHeadings;
	std::vector<double> potentialData_periods;

private:
	
	
};
#endif

