#include "PotentialVessel.h"
#include "HelpFunctions.h"
#include "CNetCage.h"


using namespace std;
using namespace Eigen;

PotentialVessel::PotentialVessel(const string& simObjectName, ISimObjectCreator* const creator) : SimObject(simObjectName){
	
#ifdef FH_VISUALIZATION
	std::string RenderString = "BOX";
	creator->GetStringParam("RenderType",RenderString,RenderString);
	
	if( RenderString == "BOX" || RenderString == "Box" || RenderString == "box")		RenderAs = BOX;
	if( RenderString == "MESH" ||RenderString == "Mesh" || RenderString == "mesh")		RenderAs = MESH;
	
	if (RenderAs == MESH) {
		creator->GetStringParam("MeshFileName",meshFileName);
		if(meshFileName.length() == 0)
			creator->ReportParameterError("MeshFileName", "Please input a mesh file name.");
	
		creator->GetStringParam("MeshFileLocation",meshFileLocation);
		if(meshFileLocation.length() == 0)
			creator->ReportParameterError("MeshFileLocation", "Please input the location of the mesh file.");
	}
#endif
	string includeWaveForces = "";
	creator->GetStringParam("IncludeWaveforces",includeWaveForces,"yes");
	if (includeWaveForces.compare("yes") == 0 || includeWaveForces.compare("Yes") == 0 || includeWaveForces.compare("YES") == 0)
		waveForces = true;
	else if(includeWaveForces.compare("Synthetic") == 0 || includeWaveForces.compare("synthetic") == 0 || includeWaveForces.compare("SYNTHETIC") == 0){
		double waveFrequency,waveHeading;
		creator->GetDoubleParam("SyntheticWaveFrequency",&waveFrequency);
		creator->GetDoubleParam("SyntheticWaveHeading",&waveHeading);
		string waveType;
		creator->GetStringParam("SyntheticWaveType",waveType);
		double forceAmplitudes[]= {0,0};
		bool typeForce = false;
		if(includeWaveForces.compare("Force") == 0 || includeWaveForces.compare("force") == 0 || includeWaveForces.compare("FORCE") == 0){
			creator->GetDoubleParamArray("ForceAmplitudes",forceAmplitudes,2);	
			typeForce = true;
		}
		syntWave = new SyntheticWave(waveFrequency,waveHeading,forceAmplitudes,typeForce);
		waveForces = false;
		syntheticWaveForces = true;
	}
	else{
		waveForces = false;
		syntheticWaveForces = false;
	}

		
	double zero[]= {0,0,0};
	creator->GetDoubleParamArray("VesselMainDimensions",vesselMainDimensions,3,zero);
	///--------------------------------------------
	/// read current TODO
	///--------------------------------------------

	for (size_t i =0;i<3 ;i++){
		currentVelocity(i) = 0;
		currentVelocity(i+3) = 0;
	}

	creator->AddInport	("ThrusterForces",		6, &m_ThrusterForces);
	creator->AddInport	("OtherForces",			6, &m_OtherForces);

	
	creator->AddOutport	("Eta"	,			6, PORT_FUNCTION(PotentialVessel::Position));
	creator->AddOutport	("Nu"	,			6, PORT_FUNCTION(PotentialVessel::VelocityBODY));
	creator->AddOutport ("NuDot",			6, PORT_FUNCTION(PotentialVessel::NuDot));
	creator->AddOutport	("FroudeKrylovForce",	6, PORT_FUNCTION(PotentialVessel::FroudeForce));
	creator->AddOutport	("MeanDriftForce"	,	6, PORT_FUNCTION(PotentialVessel::MeanWaveForce));
	creator->AddOutport	("VelocityPerturbation",6, PORT_FUNCTION(PotentialVessel::VelocityPerturbation));
	creator->AddOutport	("CurrentVelocity",6, PORT_FUNCTION(PotentialVessel::CurrentVel));

	double zero6[] = {0,0,0,0,0,0};
	indexEta	= creator->AddState("Eta",		6);
	indexNu		= creator->AddState("Nu",		6,zero6);
	indexNuDot	= creator->AddState("NuDot",	6,zero6);
	indexMeanNu = creator->AddState("NuMean",6,zero6);
	
	initializeMembers();
}

void PotentialVessel::FinalSetup( const double T, const double *const X, ISimObjectCreator* const creator ){

	environment = dynamic_cast<DeepSeaGravityWaves*>( static_cast<CEnvironment*>(creator->GetSharedResource("CSeaEnvironment") ));
	FroudeKrylov->setEnvironment(environment);
	double pos[] = {X[indexEta],X[indexEta+1]};
	seaDepth = environment->GetSeadepth(pos);
}



///**************************************
///
///    Rigid body Coriolis and centripetal forces   
///
///**************************************
void PotentialVessel::RigidBodyCoriolisForces(const double T,const double* const X){
	rigidBodyCoriolisMatrix.setZero();
	//TODO : fix name on nuRelative
	//nuRelative = nu;// - currentVelocity;
	Matrix<double,3,1> nu1; nu1<< nu(0),nu(1),nu(2);//nuRelative(0),nuRelative(1),nuRelative(2);
	Matrix<double,3,1> nu2; nu2 << nu(3),nu(4),nu(5);//nuRelative(3),nuRelative(4),nuRelative(5);

	/*Matrix3d sMatrixNu1 = Smtrx(Eigen::Vector3d(nuRelative(0),nuRelative(1),nuRelative(2)));
	Matrix3d sMatrixNu2 = Smtrx(Eigen::Vector3d(nuRelative(0),nuRelative(1),nuRelative(2)));
	*/
	//--------new (3.46)
	Matrix3d sM11v1PLUSm12v2 = Smtrx(rigidBodyMassMatrix.topLeftCorner(3,3)*nu1 + rigidBodyMassMatrix.topRightCorner(3,3)*nu2);
	Matrix3d sM21v1PLUSm22v2 = Smtrx(rigidBodyMassMatrix.bottomLeftCorner(3,3)*nu1 + rigidBodyMassMatrix.bottomRightCorner(3,3)*nu2);

	/*Matrix3d sMatrixNu1 = Smtrx(Eigen::Vector3d(nu(0),nu(1),nu(2)));
	Matrix3d sMatrixNu2 = Smtrx(Eigen::Vector3d(nu(3),nu(4),nu(5)));

	Matrix<double,3,1> r_g = Eigen::Matrix<double,3,1>(CG[0],CG[1],CG[2]);
	Matrix3d rgMatrix = r_g*r_g.transpose();
	double rgScalar = r_g.transpose()*r_g;
	Matrix3d identityMatrix = Matrix3d::Identity();
	Matrix3d inertiaBouyancy = rigidBodyMassMatrix.bottomRightCorner(3,3) - (rigidBodyMassMatrix(0,0)*rgMatrix - (rgScalar*Matrix3d::Identity()));
	
	Matrix3d skewMatrixInertiaBouyancyNu2 = Smtrx(inertiaBouyancy*nu2);*/
	
	//Matrix<double,6,6> currentRotationMatrix = Matrix<double,6,6>::Zero();
	//currentRotationMatrix.topLeftCorner(3,3) = RotationMatrix(Eigen::Vector3f(etaVector(3),etaVector(4),etaVector(5))).transpose();
		
	//rigidBodyCoriolisMatrix.topRightCorner(3,3) = -rigidBodyMassMatrix(0,0) * sMatrixNu1 - rigidBodyMassMatrix(0,0) * sMatrixNu2 * skewMatrixCG;
	//rigidBodyCoriolisMatrix.bottomLeftCorner(3,3) = -rigidBodyMassMatrix(0,0) * sMatrixNu1 + rigidBodyMassMatrix(0,0) *  skewMatrixCG * sMatrixNu2;
	//rigidBodyCoriolisMatrix.bottomRightCorner(3,3) = -skewMatrixInertiaBouyancyNu2;	

	//---------- new (4.46)
	rigidBodyCoriolisMatrix.topRightCorner(3,3) = -sM11v1PLUSm12v2;
	rigidBodyCoriolisMatrix.bottomLeftCorner(3,3) = -sM11v1PLUSm12v2;
	rigidBodyCoriolisMatrix.bottomRightCorner(3,3) = -sM21v1PLUSm22v2;


	v_rigidBodyCoriolisForce = rigidBodyCoriolisMatrix*nu;
}
	
///**************************************
///
///    Added mass Coriolis and centripetal forces     
///
///**************************************
void PotentialVessel::AddedMassCoriolisForces(const double T, const double* const X){

	//Matrix<double,3,1> nuRelative1;
	nuRelative1 << nuRelative(0),nuRelative(1),nuRelative(2);
	
	//Matrix<double,3,1> nuRelative2;
	nuRelative2 << nuRelative(3),nuRelative(4),nuRelative(5);
	
	
	addedmassCoriolisMatrix.setZero();
	addedmassCoriolisMatrix.topRightCorner(3,3) = -Smtrx(zeroFrequencyAddedMass.topLeftCorner(3,3)*nuRelative1 + zeroFrequencyAddedMass.topRightCorner(3,3)*nuRelative2);
	addedmassCoriolisMatrix.bottomLeftCorner(3,3) = addedmassCoriolisMatrix.topRightCorner(3,3); 
	addedmassCoriolisMatrix.bottomRightCorner(3,3) = -Smtrx(zeroFrequencyAddedMass.bottomLeftCorner(3,3)*nuRelative1 + zeroFrequencyAddedMass.bottomRightCorner(3,3)*nuRelative2);
	v_addedMassCoriolisForce = addedmassCoriolisMatrix*nuRelative;
	
	//cout << addedmassCoriolisMatrix << endl << "--------------"<< T << endl;
	//	cout << -(addedmassCoriolisMatrix.transpose()) << endl << "--------------"<< T <<endl;
	//addedmassCoriolisMatrix.transpose();
	//cout << (addedmassCoriolisMatrix.transpose()) << endl << "--------------"<< T <<endl;
	//addedmassCoriolisMatrix.transpose();
	//cout << (-addedmassCoriolisMatrix).transpose() << endl << "--------------"<< T <<endl;
	
}
void PotentialVessel::FluidMemoryForces(const double T, const double* const X){

	const double *forcesFluidMemory = m_OtherForces->GetPortValue(T,X);

	for (size_t i =0;i<6;i++)
		v_fluidMemoryForces(i) = forcesFluidMemory[i];


	//v_fluidMemoryForces << 0,0,0,0,0,0;
	
}
void PotentialVessel::HydroStaticRestoringForces(const double T, const double* const X){
	
	///**************************************
	///
	///    Calculating hydrostatic restoring forces
	///
	///**************************************
	
	double phi = X[indexEta+3];		//< roll angle
	double theta=X[indexEta+4];		//< pitch angle
	
	v_restoringForce(0) =  (W-B) * sin(theta);
	v_restoringForce(1) = -(W-B) * cos(theta) * sin(phi);
	v_restoringForce(2) = -(W-B) * cos(theta) * cos(phi);
	v_restoringForce(3) = -(CG[1]*W - CB[1]*B) * cos(theta) * cos(phi) + (CG[2]*W - CB[2]*B) * cos(theta) * sin(phi);
	v_restoringForce(4) =  (CG[2]*W - CB[2]*B) * sin(theta)			 + (CG[0]*W - CB[0]*B) * cos(theta) * cos(phi);
	v_restoringForce(5) = -(CG[0]*W - CB[0]*B) * cos(theta) * sin(phi) - (CG[1]*W - CB[1]*B) * sin(theta);

}
void PotentialVessel::FroudeKrylovForces(const double T, const double* const X){

	double *firstOrderWaveForce = FroudeKrylov->GetForce(T,X);
	
	for (size_t i =0;i<6;i++)
		v_FKForces(i) = firstOrderWaveForce[i];
	

}
void PotentialVessel::MeanDriftWaveForces(const double T, const double* const X){
	
	double *secondOrderWaveForce = WaveDrift->GetForce(T,X);
	
	for (size_t i =0;i<6;i++)
		v_MeanDriftForces(i) = secondOrderWaveForce[i];
}
	
///**************************************
///
///   Forces due to current and vessel rotational speed  A_c  as in Refsnes to compensate for varying incident current heading
///
///****************************
void PotentialVessel::VaryingCurrentDirectionForces(const double T, const double* const X){
	
	v_varyingCurrentDirectionForce.setZero();
	
	Matrix<double,6,6> S_c = Eigen::Matrix<double,6,6>::Zero();						
	S_c.topLeftCorner(3,3) = Smtrx(Vector3d(nu(3),nu(4),nu(5)));
	Matrix<double,6,6> tempMatrix = zeroFrequencyAddedMass * S_c;
	
	v_varyingCurrentDirectionForce = zeroFrequencyAddedMass * S_c * currentVelocity;
}

void PotentialVessel::initializeMembers(){
	v_rigidBodyCoriolisForce = Matrix<double,6,1>::Zero(); // .setZero();
	v_addedMassCoriolisForce = Matrix<double,6,1>::Zero(); // .setZero();;	
	v_fluidMemoryForces = Matrix<double,6,1>::Zero(); // .setZero();;		
	v_restoringForce = Matrix<double,6,1>::Zero(); // .setZero();;			
	v_FKForces = Matrix<double,6,1>::Zero(); // .setZero();;					
	v_MeanDriftForces = Matrix<double,6,1>::Zero(); // .setZero();;		
	v_varyingCurrentDirectionForce = Matrix<double,6,1>::Zero(); // .setZero();;	
	v_otherForces = Matrix<double,6,1>::Zero();

	nu = Matrix<double,6,1>::Zero(); // .setZero();;
	eta = Matrix<double,6,1>::Zero(); 
	currentVelocity = Matrix<double,6,1>::Zero(); // .setZero();;
	nuRelative = Matrix<double,6,1>::Zero(); // .setZero();;
	nuRelative1 = Matrix<double,3,1>::Zero(); // .setZero();;
	nuRelative2 = Matrix<double,3,1>::Zero(); // .setZero();;

	v_thrusterForces = Matrix<double,6,1>::Zero(); // .setZero();;			
	v_SUMforces = Matrix<double,6,1>::Zero(); // .setZero();;
	v_sumWOwaveFrequencyForces = Matrix<double,6,1>::Zero();
		
	rigidBodyMassMatrix = Matrix<double,6,6>::Zero(); // .setZero();;			
	rigidBodyCoriolisMatrix = Matrix<double,6,6>::Zero(); // .setZero();;	
	addedmassCoriolisMatrix = Matrix<double,6,6>::Zero(); // .setZero();;	
	zeroFrequencyAddedMass = Matrix<double,6,6>::Zero(); // .setZero();;	
	varyingCurrentHeadingMatrix = Matrix<double,6,6>::Zero(); // .setZero();;	

	skewMatrixCG = Smtrx(Eigen::Vector3d(CG[0],CG[1],CG[2]));
}


#ifdef FH_VISUALIZATION
void PotentialVessel::RenderInit( Ogre::Root* const ogreRoot, ISimObjectCreator* const creator )
{
	Ogre::SceneManager * sceneMgr = ogreRoot->getSceneManager("main");
	double Zero[3] = {1,0,0};
	m_Vessel =  sceneMgr->getRootSceneNode()->createChildSceneNode( m_SimObjectName+ "FollowNode" );
	DrawVessel(m_Vessel  ,sceneMgr);
	
	


	//std::cout << "End: " << __FUNCTION__ << std::endl;
}

void PotentialVessel::RenderUpdate( double T, const double* const X)
{
	const double* const position = Position(T,X);
	m_Vessel->setPosition(Ogre::Vector3(position[0],position[1],position[2]));
	
	double phi = position[3];
	double theta = position[4];
	double psi = position[5];
	
	double q0 = cos(phi/2)*cos(theta/2)*cos(psi/2)+sin(phi/2)*sin(theta/2)*sin(psi/2);
	double q1 = sin(phi/2)*cos(theta/2)*cos(psi/2)-cos(phi/2)*sin(theta/2)*sin(psi/2);
	double q2 = cos(phi/2)*sin(theta/2)*cos(psi/2)+sin(phi/2)*cos(theta/2)*sin(psi/2);
	double q3 = cos(phi/2)*cos(theta/2)*sin(psi/2)-sin(phi/2)*sin(theta/2)*cos(psi/2);
	Ogre::Quaternion quat = Ogre::Quaternion(q0,q1,q2,q3);
	m_Vessel->setOrientation(quat);		/// set pitch angle
	//m_WamitVessel->pitch(Ogre::Radian(position[4]));	/// set yaw angle
	//m_WamitVessel->yaw(Ogre::Radian(position[5]));		/// set roll angle
}

void PotentialVessel::DrawVessel(Ogre::SceneNode* renderNode, Ogre::SceneManager* sceneMgr)
{
	if( RenderAs == BOX )
	{
		Ogre::ManualObject* manual = sceneMgr->createManualObject();


		double l = vesselMainDimensions[0];
		double b = vesselMainDimensions[1];
		double d = vesselMainDimensions[2];
		// ship top
		manual->begin("BaseWhite",Ogre::RenderOperation::OT_TRIANGLE_FAN);
		manual->position(-0.3*l, 0.5*b,-0.5*d); manual->normal(0,0,-1);
		manual->position( 0.4*l, 0.5*b,-0.5*d); manual->normal(0,0,-1);
		manual->position( 0.7*l, 0,	   -0.5*d); manual->normal(0,0,-1);
		manual->position( 0.4*l,-0.5*b,-0.5*d); manual->normal(0,0,-1);
		manual->position(-0.3*l,-0.5*b,-0.5*d); manual->normal(0,0,-1);
		manual->end();

		// ship bottom
		manual->begin("BaseWhite",Ogre::RenderOperation::OT_TRIANGLE_FAN);
		manual->position(-0.3*l, 0.5*b, 0.5*d); manual->normal(0,0,1);
		manual->position( 0.4*l, 0.5*b, 0.5*d); manual->normal(0,0,1);
		manual->position( 0.7*l, 0,		0.5*d); manual->normal(0,0,1);
		manual->position( 0.4*l,-0.5*b, 0.5*d); manual->normal(0,0,1);
		manual->position(-0.3*l,-0.5*b, 0.5*d); manual->normal(0,0,1);
		manual->end();
	
		// ship stern
		manual->begin("BaseWhite",Ogre::RenderOperation::OT_TRIANGLE_FAN);
		//manual->normal(0,-1,0);
		manual->position(-0.3*l, 0.5*b,-0.5*d); manual->normal(-1,0,0);
		manual->position(-0.3*l,-0.5*b,-0.5*d);	manual->normal(-1,0,0); 
		manual->position(-0.3*l,-0.5*b, 0.5*d); manual->normal(-1,0,0);
		manual->position(-0.3*l, 0.5*b, 0.5*d); manual->normal(-1,0,0);
		manual->end();

		// starboard side
		manual->begin("BaseWhite",Ogre::RenderOperation::OT_TRIANGLE_FAN);
		//manual->normal(0,1,0); // not correct in bow region
		manual->position(-0.3*l, 0.5*b,-0.5*d); manual->normal(0,1,0);
		manual->position( 0.4*l, 0.5*b,-0.5*d); manual->normal(0,1,0);
		manual->position( 0.4*l, 0.5*b, 0.5*d); manual->normal(0,1,0);
		manual->position(-0.3*l, 0.5*b, 0.5*d); manual->normal(0,1,0);
		manual->end();

		// starboard bow 
		manual->begin("BaseWhite",Ogre::RenderOperation::OT_TRIANGLE_FAN);
		manual->position( 0.4*l, 0.5*b,-0.5*d); manual->normal(1,1,0);
		manual->position( 0.7*l, 0    ,-0.5*d); manual->normal(1,1,0);
		manual->position( 0.7*l, 0    , 0.5*d); manual->normal(1,1,0);
		manual->position( 0.4*l, 0.5*b, 0.5*d); manual->normal(1,1,0);
		manual->end();

		// port side
		manual->begin("BaseWhite",Ogre::RenderOperation::OT_TRIANGLE_FAN);
		manual->position(-0.3*l,-0.5*b,-0.5*d); manual->normal(0,1,0);
		manual->position( 0.4*l,-0.5*b,-0.5*d); manual->normal(0,1,0);
		manual->position( 0.4*l,-0.5*b, 0.5*d); manual->normal(0,1,0);
		manual->position(-0.3*l,-0.5*b, 0.5*d); manual->normal(0,1,0);
		manual->end();
		
		// port bow 
		manual->begin("BaseWhite",Ogre::RenderOperation::OT_TRIANGLE_FAN);
		manual->position( 0.4*l,-0.5*b,-0.5*d); manual->normal(1,1,0);
		manual->position( 0.7*l, 0    ,-0.5*d); manual->normal(1,1,0);
		manual->position( 0.7*l, 0    , 0.5*d); manual->normal(1,1,0);
		manual->position( 0.4*l,-0.5*b, 0.5*d); manual->normal(1,1,0);
		manual->end();

		renderNode->attachObject(manual);
	}
	else if(RenderAs == MESH ){
	
	Ogre::String source = meshFileLocation+meshFileName +".mesh";
 
	FILE* pFile = fopen( source.c_str(), "rb" );
	bool boolean (!pFile);
 
	struct stat tagStat;
	stat( source.c_str(), &tagStat );
	Ogre::MemoryDataStream* memstream = new Ogre::MemoryDataStream(source, tagStat.st_size, true);
	fread( (void*)memstream->getPtr(), tagStat.st_size, 1, pFile );
	fclose( pFile );
 
	Ogre::MeshPtr pMesh = Ogre::MeshManager::getSingleton().createManual("LocalMesh",Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
 

	Ogre::MeshSerializer meshSerializer;
	Ogre::DataStreamPtr stream(memstream);
	meshSerializer.importMesh(stream, pMesh.getPointer());
	
	Ogre::Entity* mesh = sceneMgr->createEntity("LocalMesh_Ent", "LocalMesh");
    
	m_Vessel = sceneMgr->getRootSceneNode()->createChildSceneNode();
    m_Vessel->attachObject(mesh);
	m_Vessel->scale(0.001,0.001,0.001);
	}
}
#endif

const double* const PotentialVessel::Position(const double T, const double* const X){

	for (size_t i = 0;i<6;i++){
		a_position[i] = X[indexEta+i];
		if (syntheticWaveForces && !syntWave->ForceBool()){
			const double * posPert = syntWave->PositionPerturbation(X+indexEta,T);
			a_position[i] += posPert[i];
		}
	}
	return a_position;
}
	
const double* const PotentialVessel::VelocityBODY(const double T, const double* const X){
	
	for (size_t i = 0;i<6;i++){
		a_velocity[i] = X[indexNu+i];
		if (syntheticWaveForces && !syntWave->ForceBool()){
			const double * velPert = syntWave->VelocityPerturbation(X+indexEta,T);
			a_velocity[i] += velPert[i];
		}
	}
	return a_velocity;
}
const double* const PotentialVessel::NuDot(const double T, const double* const X){

	for (size_t i = 0;i<6;i++){
		a_acceleration[i] = nuDot(i);
		if (syntheticWaveForces && !syntWave->ForceBool()){
			const double * accPert = syntWave->AccelerationPerturbation(X+indexEta,T);
			a_acceleration[i] += accPert[i];
		}
	}
	return a_acceleration;
}

	const double* const PotentialVessel::FroudeForce(const double T, const double* const X){

	for (int i =0;i<6;i++)
		firstOrderForces[i] = v_FKForces(i);
	
		return firstOrderForces;
	}

const double* const PotentialVessel::MeanWaveForce(const double T, const double* const X){

	for (int i =0;i<6;i++)
		secondOrderForces[i] = v_MeanDriftForces(i);
	
	return secondOrderForces;
}

const double* const PotentialVessel::VelocityPerturbation(const double T, const double* const X){

	double a_meanNu[] = {X[indexMeanNu],X[indexMeanNu+1],X[indexMeanNu+2],X[indexMeanNu+3],X[indexMeanNu+4],X[indexMeanNu+5]};
	double a_nu[]= {X[indexNu],X[indexNu+1],X[indexNu+2],X[indexNu+3],X[indexNu+4],X[indexNu+5]};
	for (size_t i =0;i<6;i++){
		nuPerturbation[i] = a_meanNu[i] - a_nu[i];
		//nuPerturbation[i] = nuPerturbation[i] - nuRelative(i);
	}
	return nuPerturbation;
}
void PotentialVessel::ThrusterForces(const double T, const double* const X){
	const double* forces = m_ThrusterForces->GetPortValue(T,X);
	
	for (size_t i = 0 ; i < 6;i++)
		v_thrusterForces(i) = forces[i];

}
const double* const PotentialVessel::CurrentVel(const double T, const double* const X){

	/*for(size_t i =0;i<6;i++)
		a_currentVel[i] = currentVelocity(i);*/
	return a_currentVel;
}