#ifndef VISCOUSFORCES
#define VISCOUSFORCES

//#include <ISimObjectCreator.h>
//#include <CFhSimMgr.h>
#include <string>
#include <list>
#include <iostream>
#include <locale>

#include "Eigen\Core"
//#include "SimObject.h"
//#include <ISimObjectCreator.h>
#include <string>

////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							ViscousForces
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Generates the linear and quadratic viscous forces
///
///					
/// \author			Mats Nvik Hval
///
///	\par
///					01.03.2012 MNH: Initial version
///
////////////////////////////////////////////////////////////////////////////////////////////////////

class ViscousForces
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	ViscousForces();
	ViscousForces(const double* const linearDampingExponentialCoefficients);
	void setDiagonalDampingMatrix(const double* const linearDiagonal, const double* const quadraticDiagonal);
	void setLinearDampingMatrix(const double* const xLinearDamping,const double* const yLinearDamping,const double* const zLinearDamping,const double* const pLinearDamping,const double* const qLinearDamping,const double* const rLinearDamping);
	void setQuadraticDampingMatrix(const double* const xQuadraticDamping,const double* const yQuadraticDamping,const double* const zQuadraticDamping,const double* const pQuadraticDamping,const double* const qQuadraticDamping,const double* const rQuadraticDamping);
	const double* const getViscousForces(const double* const VesselVelocity);

protected:
		

//	Eigen::Matrix<double,6,1> nu1;								/// First three elements of nu
//	Eigen::Matrix<double,6,1> nu2;								/// Last three elements of nu
	
	/*double xLinear[6];									/// the elements in the first row in the 6DOF linear damping matrix
	double yLinear[6];									/// the elements in the second row in the 6DOF linear damping matrix
	double zLinear[6];									/// the elements in the third row in the 6DOF linear damping matrix
	double pLinear[6];									/// the elements in the forth row in the 6DOF linear damping matrix
	double qLinear[6];									/// the elements in the fifth row in the 6DOF linear damping matrix
	double rLinear[6];									/// the elements in the sixth row in the 6DOF linear damping matrix
									
	double xQuadratic[6];								/// the elements in the first row in the 6DOF quadratic damping matrix
	double yQuadratic[6];								/// the elements in the second row in the 6DOF quadratic damping matrix
	double zQuadratic[6];								/// the elements in the third row in the 6DOF quadratic damping matrix
	double pQuadratic[6];								/// the elements in the forth row in the 6DOF quadratic damping matrix
	double qQuadratic[6];								/// the elements in the fifth row in the 6DOF quadratic damping matrix
	double rQuadratic[6];								/// the elements in the sixth row in the 6DOF quadratic damping matrix*/

	double linearForces[6];								/// vector with the linear forces
	double quadraticForces[6];							/// vector with the quadratic forces
	double viscousForces[6];							/// vector with the sum of linear and quadratic forces

private:
	const double* const LinearDamping(const double* const VesselVelocity);
	const double* const QuadraticDamping(const double* const VesselVelocity);
	void initializeMembers();

	double linearDampingExponentialCoeff_k;
	double linearDampingExponentialCoeff_p;
	double linearDampingExponentialCoeff_r;
	
	Eigen::Matrix<double,6,1> v_linearForces;			/// vector with the linear forces
	Eigen::Matrix<double,6,1> v_quadraticForces;			/// vector with the quadratic forces
	Eigen::Matrix<double,6,1> v_viscousForces;			/// vector with the sum of linear and quadratic forces
	Eigen::Matrix<double,6,1> nu;									/// Rigid body velocity vector
	
	Eigen::Matrix<double,6,6> quadraticDampingMatrix;	/// Matrix containing the quadratic damping terms
	Eigen::Matrix<double,6,6> linearDampingMatrix;		/// Matrix containing the quadratic damping terms


};
#endif