#include "WamitData.h"
#include "WamitReader.h"

using namespace std;
using namespace Eigen;

////////////////////////////////////////////////////////////////////////////////////////////////////
//==================================================================================================
// FUNCTION NAME:		TokenizeString
//==================================================================================================
//
///	Takes a string and returns the elements/words in a string vector removing the white spaces
///	
/// \param [in]		&line			string containing the line to tokenize
///
/// \param [in]		&tokens			an empty vector for the tokens in "line".
///
////////////////////////////////////////////////////////////////////////////////////////////////////
//void WamitReader::TokenizeString(const string &line,vector<string> &tokens)
//{
//	tokens.clear();
//	string::size_type lastPos = line.find_first_not_of(" ",0);
//	string::size_type pos = line.find_first_of(" ",lastPos);
//
//	while (string::npos != pos || string::npos !=lastPos){
//		tokens.push_back(line.substr(lastPos,pos-lastPos));
//		lastPos = line.find_first_not_of(" ",pos);
//		pos = line.find_first_of(" ",lastPos);
//	}
//}


void WamitData::Read( std::string &fname )
{
	/// read wamit vessel parameters. 
	Matrix<double,6,1> scaleDiag;
	scaleDiag << 1,-1,-1,1,-1,-1; 
	const Matrix<double,6,6> WAMIT2NED = scaleDiag.asDiagonal(); // transformation matrix for  converting from WAMIT reference frame to NED frame
	float VOLUMES[3];
	Matrix<double,6,6> wamitCmatrix;
	wamitCmatrix.setZero(); // = MatrixXf::Zero(6,6);
	ifstream file;
	float GRAVITY,ULEN,WATERDEPTH,RHO;
	string line;
	vector<string> tokens;


	file.open(fname + (".out"));
	while ( file.good() ){
		line.clear();
		getline(file,line);
		
		if(line.find("Gravity:",0) != -1){
			TokenizeString(line,tokens);
			GRAVITY = double(atof(tokens.at(1).c_str()));
			ULEN = double(atof(tokens.at(4).c_str()));
		}
		if(line.find("Water depth",0) != -1){
			TokenizeString(line,tokens);
			WATERDEPTH = double(atof(tokens.at(2).c_str()));
			RHO = double(atof(tokens.at(5).c_str()));
		}
		if(line.find("XBODY",0) != -1){
			TokenizeString(line,tokens);
			XBODY[0] = double(atof(tokens.at(2).c_str()));
			XBODY[1] = double(atof(tokens.at(5).c_str()));
			XBODY[2] = double(atof(tokens.at(8).c_str()));
			XBODY[3] = double(atof(tokens.at(11).c_str()));
		}
		if(line.find("Volumes",0) != -1){
			TokenizeString(line,tokens);
			VOLUME = double(atof(tokens.at(2).c_str())) + double(atof(tokens.at(3).c_str())) + double(atof(tokens.at(4).c_str())) /3;
		}
		if(line.find("Center of Buoyancy",0) != -1){
			TokenizeString(line,tokens);
			CB[0] = double(atof(tokens.at(4).c_str()));
			LCG = CB[0];
			CB[1] = -double(atof(tokens.at(5).c_str()));
			CB[2] = -double(atof(tokens.at(6).c_str()));
			VCG = CB[2];
		}
		if(line.find("C(3,3)") != -1){
			TokenizeString(line,tokens);
			wamitCmatrix(2,2) = double(atof(tokens.at(1).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,2);
			wamitCmatrix(2,3) = double(atof(tokens.at(2).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,3);
			wamitCmatrix(2,4) = double(atof(tokens.at(3).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,3);
		}
		if(line.find("C(4,4)") != -1){
			TokenizeString(line,tokens);
			wamitCmatrix(3,3) = double(atof(tokens.at(1).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
			wamitCmatrix(3,4) = double(atof(tokens.at(2).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
			wamitCmatrix(3,5) = double(atof(tokens.at(3).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
		}
		if(line.find("C(5,5)") != -1){
				TokenizeString(line,tokens);
			wamitCmatrix(4,4) = double(atof(tokens.at(1).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
			wamitCmatrix(4,5) = double(atof(tokens.at(2).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
		}
		if(line.find("Center of Gravity") != -1){
				TokenizeString(line,tokens);
			CG[0] = double(atof(tokens.at(4).c_str()));	
			LCG = CG[0];
			CG[1] = -double(atof(tokens.at(5).c_str()));
			CG[2] = -double(atof(tokens.at(6).c_str()));
			VCG = CG[2];
		}
		tokens.clear();
	}
	file.close();
	wamitCmatrix = WAMIT2NED*wamitCmatrix*WAMIT2NED;
	C = wamitCmatrix;
	
	/*vector< Matrix<double,6,6> > frequencyVector;
		frequencyVector.push_back(wamitCmatrix);
	vector<vector<Matrix<double,6,6>>> headingVector;
	headingVector.push_back(frequencyVector);
	C.push_back(headingVector);
	*/
	/*wamitCmatrix*/

/// Read mass matrix
	
		float mass = VOLUME*RHO;
		//Matrix<double,6,1> scaleDiag(6);
		//scaleDiag << 1,-1,-1,1,-1,-1; 
		//const Matrix<double,6,6> WAMIT2NED = scaleDiag.asDiagonal(); // transformation matrix for  converting from WAMIT reference frame to NED frame
		//cout << WAMIT2NED << endl;
		Mrbeig.setZero();// = MatrixXf::Zero(6,6);
		Matrix3f radiiOfGyration = Matrix3f::Zero(3,3);
		bool frcAlternativeTwo = false;
		bool yaw = false;
		//string line;
	//	vector<string> tokens;
		int lineNumber=0;
		file.open(fname + ".cfg");
		//vector<string> tokens;

		while(file.good()){
			getline(file,line);
			if (line.find("IALTFRC") != -1)
				for (size_t i = 0;i<line.length();i++)
					if (line[i] == '2') {
						frcAlternativeTwo = true; 
					}
		}
		file.close();

		file.open(fname +".frc");
		int dof = 0;
		while (file.good())
		{
			getline(file,line);
			lineNumber++;
			if (frcAlternativeTwo && lineNumber > 4 && 6-dof > 0){
				TokenizeString(line,tokens);
				if (tokens.size() >= 6){
					Mrbeig(dof,0) = double(atof(tokens.at(0).c_str()));
					Mrbeig(dof,1) = double(atof(tokens.at(1).c_str()));
					Mrbeig(dof,2) = double(atof(tokens.at(2).c_str()));
					Mrbeig(dof,3) = double(atof(tokens.at(3).c_str()));
					Mrbeig(dof,4) = double(atof(tokens.at(4).c_str()));
					Mrbeig(dof,5) = double(atof(tokens.at(5).c_str()));
					dof++;
				}
			}
			if(!frcAlternativeTwo) {
				TokenizeString(line,tokens);
				if(lineNumber >2 && tokens.size() >=3 && tokens.size() < 6){
					radiiOfGyration(dof,0) = double(atof(tokens.at(0).c_str()));
					radiiOfGyration(dof,1) = double(atof(tokens.at(1).c_str()));
					radiiOfGyration(dof,2) = double(atof(tokens.at(2).c_str()));
					dof++;
				}
			}
		}
		file.close();
		if (!frcAlternativeTwo){
			CG = CB;
			Mrbeig(0,0) = mass;
			Mrbeig(1,1) = mass;
			Mrbeig(2,2) = mass;
		
			Mrbeig(0,4) = mass*CG[2];
			Mrbeig(0,5) = -mass*CG[1];
			Mrbeig(1,3) = -mass*CG[2];
			Mrbeig(1,5) = mass*CG[1];
			Mrbeig(2,3) = mass*CG[1];
			Mrbeig(2,4) = -mass*CG[0];
		
			Mrbeig(3,1) = Mrbeig(1,3);
			Mrbeig(3,2) = Mrbeig(2,3);
			Mrbeig(4,0) = Mrbeig(0,4);
			Mrbeig(4,2) = Mrbeig(2,4);
			Mrbeig(5,0) = Mrbeig(0,5);
			Mrbeig(5,1) = Mrbeig(1,5);

			Mrbeig(3,3) = mass*radiiOfGyration(0,0)*abs(radiiOfGyration(0,0));
			Mrbeig(3,4) = mass*radiiOfGyration(0,1)*abs(radiiOfGyration(0,1));
			Mrbeig(3,5) = mass*radiiOfGyration(0,2)*abs(radiiOfGyration(0,2));
			Mrbeig(4,3) = mass*radiiOfGyration(1,0)*abs(radiiOfGyration(1,0));
			Mrbeig(4,4) = mass*radiiOfGyration(1,1)*abs(radiiOfGyration(1,1));
			Mrbeig(4,5) = mass*radiiOfGyration(1,2)*abs(radiiOfGyration(1,2));
			Mrbeig(5,3) = mass*radiiOfGyration(2,0)*abs(radiiOfGyration(2,0));
			Mrbeig(5,4) = mass*radiiOfGyration(2,1)*abs(radiiOfGyration(2,1));
			Mrbeig(5,5) = mass*radiiOfGyration(2,2)*abs(radiiOfGyration(2,2));
		}
	
	
		Mrbeig = WAMIT2NED*Mrbeig*WAMIT2NED;	// transform from WAMIT-axes to NED-frame.
	
	//// Read added mass and damping
	//Matrix<float,6,1> scaleDiag;
	//scaleDiag << 1,-1,-1,1,-1,-1; 
	//Matrix<float,6,6> WAMIT2NED;
	//const Matrix<float,6,6> WAMIT2NED = scaleDiag.asDiagonal(); // transformation matrix for  converting from WAMIT reference frame to NED frame
	file.open(fname + ".1");
	//int nPer = 0;
	vector<double> vec_line; // vector containing the numbers read from current line
	vector<double> wavePeriod;

	vector<Eigen::Matrix<double,6,6>, Eigen::aligned_allocator< Eigen::Matrix< double, 6, 6 > > > AfrequencyVector;
	
	vector<vector<Eigen::Matrix<double,6,6>, Eigen::aligned_allocator< Eigen::Matrix< double, 6, 6 > >>> Aheadingvector;
	Aheadingvector.push_back(AfrequencyVector);
	A.push_back(Aheadingvector);
		
	while ( file.good() ){
		getline(file,line);
		if (line.length() >5) {
			TokenizeString(line,tokens);
			for (size_t i = 0;i<tokens.size();i++)
				vec_line.push_back(atoi(tokens[i].c_str()));
				
				if (vec_line[1] == vec_line[2] && vec_line[1] == 1) {
					if (!A.empty()){
						Matrix<double,6,6> transposedMatrix = A.back().back().back().transpose();
						A.back().back().back() = 0.5 * (A.back().back().back() + transposedMatrix);
					}
					wavePeriod.push_back(vec_line[0]);
					Matrix<double,6,6> temp,temp2;
					temp.setZero();
					temp2.setZero();

					A.back().back().push_back(MatrixXd::Zero(6,6));
					B.back().back().push_back(MatrixXd::Zero(6,6));
				//	nPer++;
				}
			if (vec_line[1] >3){
				if (vec_line[2] > 3){
					vec_line[3] *= pow(ULEN,5)*RHO;
					if (vec_line.size() == 5) vec_line[4] *= pow(ULEN,5)*RHO*vec_line[0];
				}
				else {
					vec_line[3] *= pow(ULEN,4)*RHO;
					if (vec_line.size() == 5) vec_line[4] *= pow(ULEN,4)*RHO*vec_line[0];
				}
			}
			else {
				if (vec_line[2] > 3){
					vec_line[3] *= pow(ULEN,4)*RHO;
					if (vec_line.size() == 5) vec_line[4] *= pow(ULEN,4)*RHO*vec_line[0];
				}
				else {
					vec_line[3] *= pow(ULEN,3)*RHO;
					if (vec_line.size() == 5) vec_line[4] *= pow(ULEN,3)*RHO*vec_line[0];
				}
			}
			A.back().back().back()(int(vec_line[1])-1,int(vec_line[2])-1) = double(vec_line[3]);
			if (vec_line.size() == 5) B.back().back().back()(int(vec_line[1])-1,int(vec_line[2])-1) = double(vec_line[4]);
			vec_line.clear();
		}
	}
	file.close();

	for (int i = 0;i<wavePeriod.size();i++){ //nPer;
		A.back().back()[i] = WAMIT2NED*A.back().back()[i]*WAMIT2NED; //transform added mass matrix to NED-frame
		B.back().back()[i] = WAMIT2NED*B.back().back()[i]*WAMIT2NED; // transform potential damping matrix to NED frame
	}
}
