////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							FileReader
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Reads input and output files from WAMIT.
///
///					
/// \author			Mats Nvik Hval
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef WAMITREADER_H
#define WAMITREADER_H

#include <Eigen\Dense>
#include <iostream>
#include <fstream>
//#include <sstream>
#include <vector>
#include <Eigen\Core>
#include <stdio.h>
#include <string>
#include <string.h>
#include <cstring>
#include "HelpFunctions.h"
//#include <algorithm>

class WamitReader
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	void AddedMassAndDamping(std::vector<double> *wavePeriods,std::vector<Eigen::Matrix<float,6,6>, Eigen::aligned_allocator<Eigen::Matrix<float,6,6>>> *addedMass, std::vector<Eigen::Matrix<float,6,6>, Eigen::aligned_allocator<Eigen::Matrix<float,6,6>>> *potentialDamping, std::string fileName, const float ULEN, const float RHO);
	void RAOdata(std::vector<double> *wavePeriods,std::vector<double> *waveHeadings,std::vector<std::vector<Eigen::MatrixXf>> *RAOdata, std::string fileName, const float ULEN, const float RHO, const std::string wamitOption, const float GRAVITY);
	void VesselParameters(std::string fileName, float *ULEN, float *RHO, float *GRAVITY, float *WATERDEPTH, float XBODY[], float *VOLUME, float COB[], Eigen::Matrix<float,6,6> &wamitCmatrix, float COG[]);
	void MeanDrift(std::string fileName, std::vector<float> *secondWaveHeadings,std::vector<std::vector<std::vector<Eigen::MatrixXf>>> *meanDriftData,const float ULEN, const float RHO,const float GRAVITY, const std::string wamitOption);
	void MassMatrix(const std::string fileName, const float RHO, const float VOLUME,const float COG[], const float COB[], Eigen::Matrix<float,6,6> &rigidBodyMassMatrix);

private:
	
	std::string fileName;																///< Base name of the vessel/file from wamit. i.e "vessel" in "vessel.1"
	std::ifstream file;					
	//void TokenizeString(const std::string &line,std::vector<std::string> &tokens);		///< Function to tokenize a string into a vector.
	
};

#endif
