﻿#include "PotentialVessel.h"
#include "WamitVessel.h"
#include "HelpFunctions.h"
#include <iostream>
#include "Eigen/StdVector"
#include <CNetCage.h>

//#include <fstream>

using namespace Eigen;
using namespace std;

WamitVessel::WamitVessel(const string& simObjectName, ISimObjectCreator* const creator): PotentialVessel(simObjectName,creator){

	double linearDampingDiagonal[6];	double quadraticDampingDiagonal[6];
	double uLinearDamping[6];			double vLinearDamping[6];		double wLinearDamping[6]; 
	double pLinearDamping[6];			double qLinearDamping[6];		double rLinearDamping[6];
	double uQuadraticDamping[6];		double vQuadraticDamping[6]; 	double wQuadraticDamping[6];
	double pQuadraticDamping[6];		double qQuadraticDamping[6]; 	double rQuadraticDamping[6];
	double linearDampingExponentialCoefficients[] = {0.1,15,3};

	std::string dampingMatrixType; bool diagonalDampingMatrix = false;
	creator->GetStringParam("PotentialFileName",potentialFileName);
	creator->GetStringParam("PotentialFileLocation",potentialFileLocation);
	creator->GetStringParam("DampingMatrixInputType",dampingMatrixType,"diagonal");
	if (dampingMatrixType.compare("diagonal") == 0 || dampingMatrixType.compare("Diagonal") == 0 || dampingMatrixType.compare("DIAGONAL") == 0){
		diagonalDampingMatrix = true;
		creator->GetDoubleParamArray("LinearDampingDiagonal",linearDampingDiagonal,6);
		creator->GetDoubleParamArray("QuadraticDampingDiagonal",quadraticDampingDiagonal,6);		
	}
	else{
		creator->GetDoubleParamArray("RowOneLinearDamping",uLinearDamping,6);
		creator->GetDoubleParamArray("RowTwoLinearDamping",vLinearDamping,6);
		creator->GetDoubleParamArray("RowThreeLinearDamping",wLinearDamping,6);
		creator->GetDoubleParamArray("RowFourLinearDamping",pLinearDamping,6);
		creator->GetDoubleParamArray("RowFiveLinearDamping",qLinearDamping,6);
		creator->GetDoubleParamArray("RowSixLinearDamping",rLinearDamping,6);

		creator->GetDoubleParamArray("RowOneQuadraticDamping",uQuadraticDamping,6);
		creator->GetDoubleParamArray("RowTwoQuadraticDamping",vQuadraticDamping,6);
		creator->GetDoubleParamArray("RowThreeQuadraticDamping",wQuadraticDamping,6);
		creator->GetDoubleParamArray("RowFourQuadraticDamping",pQuadraticDamping,6);
		creator->GetDoubleParamArray("RowFiveQuadraticDamping",qQuadraticDamping,6);
		creator->GetDoubleParamArray("RowSixQuadraticDamping",rQuadraticDamping,6);
	}

	
	creator->GetDoubleParamArray("linearDampingExponentialCoefficients",linearDampingExponentialCoefficients,3,linearDampingExponentialCoefficients);
	
	viscousForce = new ViscousForces(linearDampingExponentialCoefficients);
	
	FroudeKrylov = new FKForce(potentialFileName,potentialFileLocation);
	WaveDrift = new MeanDriftForce(potentialFileName, potentialFileLocation);
	
	if (diagonalDampingMatrix)
		viscousForce->setDiagonalDampingMatrix(linearDampingDiagonal,quadraticDampingDiagonal);
	else{
		viscousForce->setLinearDampingMatrix(uLinearDamping,vLinearDamping,wLinearDamping,pLinearDamping,qLinearDamping,rLinearDamping);
		viscousForce->setQuadraticDampingMatrix(uQuadraticDamping,vQuadraticDamping,wQuadraticDamping,pQuadraticDamping,qQuadraticDamping,rQuadraticDamping);
	}

	v_viscousForces = Matrix<double,6,1>::Zero();

	creator->GetStringParam("PotentialFileName",potentialFileName);
	if(potentialFileName.length() == 0)
		creator->ReportParameterError("PotentialFileName", "Must correspond to the output files from WAMIT.");
	
	creator->GetStringParam("PotentialFileLocation",potentialFileLocation);
	if(potentialFileLocation.length() == 0)
		creator->ReportParameterError("PotentialFileLocation", "The directory for the WAMIT files is incorrect.");
	
	Read();
	//reads vessel data from wamit files. Rigid body mass matrix, center of gravity and bouyancy, volume, water density, gravity, 
}
WamitVessel::~WamitVessel(){
	delete viscousForce;
	delete WaveDrift;
	delete FroudeKrylov;

}

void WamitVessel::OdeFcn(const double T, const double* const X, double* const XDot, const bool IsMajorTimeStep){
	//TODO fix current retrievement
	
	double pos[] = {X[indexEta],X[indexEta+1],X[indexEta+2]};
	double currentSpeed[]= {0,0,0};
	environment->GetCurrentVelocity(T,X,pos,currentSpeed);
	currentVelocity << currentSpeed[0],currentSpeed[1],currentSpeed[2],0,0,0;
	for (size_t i =0;i<6;i++)
		a_currentVel[i] = currentVelocity(i);
	eta << X[indexEta],X[indexEta + 1],X[indexEta + 2],X[indexEta + 3],X[indexEta + 4],X[indexEta + 5];
	nu << X[indexNu],X[indexNu+1],X[indexNu+2],X[indexNu+3],X[indexNu+4],X[indexNu+5];
	
	Eigen::Matrix<double,6,6> rotMatrix = Eigen::Matrix<double,6,6>::Zero();
	Eigen::Matrix<double,3,3> R = RotationMatrix(Eigen::Vector3d(X[indexEta+3],X[indexEta+4],X[indexEta+5]));
	rotMatrix.topLeftCorner(3,3) = R.transpose();
	currentVelocity = rotMatrix*currentVelocity;
	nuRelative = nu - currentVelocity;
	//std::cout << currentVelocity << std::endl << "-------------------" << std::endl;
	
	ThrusterForces(T,X);
	RigidBodyCoriolisForces(T,X);
	AddedMassCoriolisForces(T,X);
	if (waveForces){
		FluidMemoryForces(T,X);
		MeanDriftWaveForces(T,X);
		FroudeKrylovForces(T,X);
	}
	HydroStaticRestoringForces(T,X);
	VaryingCurrentDirectionForces(T,X);
	getViscousForces(T,X);

	v_SUMforces = v_varyingCurrentDirectionForce - v_addedMassCoriolisForce - v_rigidBodyCoriolisForce - v_viscousForces - v_restoringForce + v_thrusterForces;
	
	nuDotMean = mInv* (v_SUMforces);
	
	v_SUMforces = v_SUMforces + v_FKForces + v_fluidMemoryForces+ v_MeanDriftForces ;
	
	//nuDotMean = mInv * (v_SUMforces + v_FKForces - v_fluidMemoryForces) - (mInv * v_SUMforces);
	
	nuDot = mInv *v_SUMforces;
	etaDot = Jmatrix(X+indexEta) * nu;

	for (size_t i = 0 ; i<6 ; i++){
		XDot[indexMeanNu+i]= nuDotMean(i);
		XDot[indexNu + i] = nuDot(i);
		XDot[indexEta +i] = etaDot(i);
		//eta(i) = X[indexEta+i];
	}
}

void WamitVessel::Read(){

	Matrix<double,6,1> scaleDiag;
	scaleDiag << 1,-1,-1,1,-1,-1; 
	const Matrix<double,6,6> WAMIT2NED = scaleDiag.asDiagonal(); // transformation matrix for  converting from WAMIT reference frame to NED frame
	
	double VOLUMES[3];
	Matrix<double,6,6> wamitCmatrix = Matrix<double,6,6>::Zero();
	double ULEN;

	string fileName = potentialFileLocation + potentialFileName;
	ifstream file(fileName + ".out");
	string line;
	vector<string> tokens;
	while ( file.good() ){
		line.clear();
		getline(file,line);
		if(line.find("Gravity:",0) != -1){
			TokenizeString(line,tokens);
			GRAVITY = double(atof(tokens.at(1).c_str()));
			ULEN = double(atof(tokens.at(4).c_str()));
		}
		if(line.find("Water depth",0) != -1){
			TokenizeString(line,tokens);
			RHO = double(atof(tokens.at(5).c_str()));
		}
		if(line.find("Volumes",0) != -1){
			TokenizeString(line,tokens);
			VOLUMES[0] = double(atof(tokens.at(2).c_str()));
			VOLUMES[1] = double(atof(tokens.at(3).c_str()));
			VOLUMES[2] = double(atof(tokens.at(4).c_str()));
		}
		if(line.find("Center of Buoyancy",0) != -1){
			TokenizeString(line,tokens);
			CB[0] = double(atof(tokens.at(4).c_str()));
			CB[1] = -double(atof(tokens.at(5).c_str()));
			CB[2] = -double(atof(tokens.at(6).c_str()));
		}
		if(line.find("C(3,3)") != -1){
			TokenizeString(line,tokens);
			wamitCmatrix(2,2) = double(atof(tokens.at(1).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,2);
			wamitCmatrix(2,3) = double(atof(tokens.at(2).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,3);
			wamitCmatrix(2,4) = double(atof(tokens.at(3).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,3);
		}
		if(line.find("C(4,4)") != -1){
			TokenizeString(line,tokens);
			wamitCmatrix(3,3) = double(atof(tokens.at(1).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
			wamitCmatrix(3,4) = double(atof(tokens.at(2).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
			wamitCmatrix(3,5) = double(atof(tokens.at(3).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
		}
		if(line.find("C(5,5)") != -1){
				TokenizeString(line,tokens);
			wamitCmatrix(4,4) = double(atof(tokens.at(1).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
			wamitCmatrix(4,5) = double(atof(tokens.at(2).c_str()))*(RHO)*(GRAVITY)*pow(ULEN,4);
		}
		if(line.find("Center of Gravity") != -1){
				TokenizeString(line,tokens);
			CG[0] = double(atof(tokens.at(4).c_str()));			
			CG[1] = -double(atof(tokens.at(5).c_str()));
			CG[2] = -double(atof(tokens.at(6).c_str()));
		}
		tokens.clear();
	}
	file.close();
	VOLUME = (VOLUMES[0] + VOLUMES[1] + VOLUMES[2]) /3;
	wamitCmatrix = WAMIT2NED*wamitCmatrix*WAMIT2NED;

	for (size_t i= 0;i<6;i++){
		for(size_t j=0;j<6;j++){
			potentialC[i][j] = wamitCmatrix(i,j);
		}
	}

	skewMatrixCG = Smtrx(Eigen::Vector3d(CG[0],CG[1],CG[2]));

	//read mass matrix;
	
	double mass = VOLUME*RHO;
		
	rigidBodyMassMatrix.setZero();// = MatrixXf::Zero(6,6);
	Matrix3d radiiOfGyration = Matrix3d::Zero(3,3);
	bool frcAlternativeTwo = false;
	bool yaw = false;
	int lineNumber=0;
	file.open(fileName + ".cfg");
		//vector<string> tokens;
	while(file.good()){
		getline(file,line);
		if (line.find("IALTFRC") != -1)
			for (size_t i = 0;i<line.length();i++)
				if (line[i] == '2') {
					frcAlternativeTwo = true; 
				}
	}
	
	file.close();
	file.open(fileName +".frc");
	int dof = 0;
	while (file.good()){
		getline(file,line);
		lineNumber++;
		if (frcAlternativeTwo && lineNumber > 4 && 6-dof > 0){
			TokenizeString(line,tokens);
			if (tokens.size() >= 6){
				rigidBodyMassMatrix(dof,0) = double(atof(tokens.at(0).c_str()));
				rigidBodyMassMatrix(dof,1) = double(atof(tokens.at(1).c_str()));
				rigidBodyMassMatrix(dof,2) = double(atof(tokens.at(2).c_str()));
				rigidBodyMassMatrix(dof,3) = double(atof(tokens.at(3).c_str()));
				rigidBodyMassMatrix(dof,4) = double(atof(tokens.at(4).c_str()));
				rigidBodyMassMatrix(dof,5) = double(atof(tokens.at(5).c_str()));
				dof++;
			}
		}
		if(!frcAlternativeTwo) {
			TokenizeString(line,tokens);
			if(lineNumber >2 && tokens.size() >=3 && tokens.size() < 6){
				radiiOfGyration(dof,0) = double(atof(tokens.at(0).c_str()));
				radiiOfGyration(dof,1) = double(atof(tokens.at(1).c_str()));
				radiiOfGyration(dof,2) = double(atof(tokens.at(2).c_str()));
				dof++;
			}
		}
	}
	file.close();
	if (!frcAlternativeTwo){
		CG[0] = CB[0];
		CG[1] = CB[1];
		CG[2] = CB[2];
		rigidBodyMassMatrix(0,0) = mass;
		rigidBodyMassMatrix(1,1) = mass;
		rigidBodyMassMatrix(2,2) = mass;
	
		rigidBodyMassMatrix(0,4) = mass*CG[2];
		rigidBodyMassMatrix(0,5) = -mass*CG[1];
		rigidBodyMassMatrix(1,3) = -mass*CG[2];
		rigidBodyMassMatrix(1,5) = mass*CG[1];
		rigidBodyMassMatrix(2,3) = mass*CG[1];
		rigidBodyMassMatrix(2,4) = -mass*CG[0];
	
		rigidBodyMassMatrix(3,1) = rigidBodyMassMatrix(1,3);
		rigidBodyMassMatrix(3,2) = rigidBodyMassMatrix(2,3);
		rigidBodyMassMatrix(4,0) = rigidBodyMassMatrix(0,4);
		rigidBodyMassMatrix(4,2) = rigidBodyMassMatrix(2,4);
		rigidBodyMassMatrix(5,0) = rigidBodyMassMatrix(0,5);
		rigidBodyMassMatrix(5,1) = rigidBodyMassMatrix(1,5);
		
		rigidBodyMassMatrix(3,3) = mass*radiiOfGyration(0,0)*abs(radiiOfGyration(0,0));
		rigidBodyMassMatrix(3,4) = mass*radiiOfGyration(0,1)*abs(radiiOfGyration(0,1));
		rigidBodyMassMatrix(3,5) = mass*radiiOfGyration(0,2)*abs(radiiOfGyration(0,2));
		rigidBodyMassMatrix(4,3) = mass*radiiOfGyration(1,0)*abs(radiiOfGyration(1,0));
		rigidBodyMassMatrix(4,4) = mass*radiiOfGyration(1,1)*abs(radiiOfGyration(1,1));
		rigidBodyMassMatrix(4,5) = mass*radiiOfGyration(1,2)*abs(radiiOfGyration(1,2));
		rigidBodyMassMatrix(5,3) = mass*radiiOfGyration(2,0)*abs(radiiOfGyration(2,0));
		rigidBodyMassMatrix(5,4) = mass*radiiOfGyration(2,1)*abs(radiiOfGyration(2,1));
		rigidBodyMassMatrix(5,5) = mass*radiiOfGyration(2,2)*abs(radiiOfGyration(2,2));
	}
	tokens.clear();
	// transform from WAMIT-axes to NED-frame.
	rigidBodyMassMatrix = WAMIT2NED*rigidBodyMassMatrix*WAMIT2NED;

	W = rigidBodyMassMatrix(0,0)*GRAVITY;
	
	
	//read added mass and damping data

	vector<double> wamitWavePeriods;

	std::vector<Eigen::Matrix<double,6,6>, Eigen::aligned_allocator< Eigen::Matrix< double, 6, 6 > >> addedMass;
	std::vector<Eigen::Matrix<double,6,6>, Eigen::aligned_allocator< Eigen::Matrix< double, 6, 6 > >> potentialDamping;
	line.clear();
	addedMass.clear();
	potentialDamping.clear();
	file.open(fileName+".1");
//	int indexOfFirstWhiteSpace;
	//	vector<double> vec_line; // vector containing the numbers read from current line
	string str_value;
	while ( file.good() ){
		tokens.clear();
		line.clear();
		getline(file,line);
		TokenizeString(line,tokens);
		if(tokens.size() > 0){
			if (tokens[1].compare(tokens[2]) == 0 && atoi(tokens[1].c_str()) == 1){
				const char *wavePer = tokens[0].c_str();
				wamitWavePeriods.push_back(atof(wavePer));
				addedMass.push_back(Matrix<double,6,6>::Zero());
				potentialDamping.push_back(Matrix<double,6,6>::Zero());
			}
			int dof1 = atoi(tokens[1].c_str());
			int dof2 = atoi(tokens[2].c_str());
			double addedMassValue = atof(tokens[3].c_str());
			double potentialDampingValue = 0;
			double potentialFrequency = 0;	
			if (tokens.size() == 5){
				potentialDampingValue = atof(tokens[3].c_str());
				potentialFrequency = 2*M_PI / wamitWavePeriods.back();
			}
				/*if (tokens[1].compare(tokens[2]) == 0 && int(tokens[1]) == 1) {
					if (!addedMass->empty()){
						Matrix<float,6,6> transposedMatrix = addedMass->back().transpose();
						addedMass->back() = 0.5 * (addedMass->back() + transposedMatrix);
					}
					wavePeriods->push_back(vec_line[0]);
					Matrix<float,6,6> temp,temp2;
					temp.setZero();
					temp2.setZero();
					addedMass->push_back(temp);
					potentialDamping->push_back(temp2);
					nPer++;
				}*/
			if (dof1 >3){
				if (dof2 > 3){
					addedMass.back()(dof1-1,dof2-1) = addedMassValue * pow(ULEN,5)*RHO; //pow(ULEN,5)*RHO;
				if (tokens.size() == 5) 
					potentialDamping.back()(dof1-1,dof2-1) = potentialDampingValue* pow(ULEN,5)*RHO*potentialFrequency;
			}
			else {
				addedMass.back()(dof1-1,dof2-1) = addedMassValue * pow(ULEN,4)*RHO;
				if (tokens.size() == 5) 
					potentialDamping.back()(dof1-1,dof2-1) = potentialDampingValue * pow(ULEN,4)*RHO*potentialFrequency;
			}
			}
			else {
				if (dof2 > 3){
					addedMass.back()(dof1-1,dof2-1) = addedMassValue * pow(ULEN,4)*RHO;
					if (tokens.size() == 5) 
						potentialDamping.back()(dof1-1,dof2-1) = potentialDampingValue * pow(ULEN,4)*RHO*potentialFrequency;
				}
				else {
					addedMass.back()(dof1-1,dof2-1) = addedMassValue * pow(ULEN,3)*RHO;
					if (tokens.size() == 5) 
						potentialDamping.back()(dof1-1,dof2-1) = potentialDampingValue * pow(ULEN,3)*RHO*potentialFrequency;
				}
			}
		}
	}
	
	file.close();
	

	for (int i = 0;i<wamitWavePeriods.size();i++){
		Matrix<double,6,6> temp = addedMass.at(i);
		Matrix<double,6,6> transposed = temp.transpose();
		addedMass.at(i) = WAMIT2NED*(0.5*(temp+transposed))*WAMIT2NED; //transform added mass matrix to NED-frame
		temp = potentialDamping.at(i);
		transposed = temp.transpose();
		potentialDamping.at(i)= WAMIT2NED*(0.5*(temp+transposed))*WAMIT2NED; // transform potential damping matrix to NED frame
	}
//	cout << addedMass.at(0) << endl;

	int indexOfZeroFrequency= -1; /// Zero frequency == inf period -> waveperiod = -1
	for (size_t i = 0 ; i< wamitWavePeriods.size();i++){
		if (wamitWavePeriods.at(i) == -1 || (wamitWavePeriods.at(i) > 1.01 *-1 && wamitWavePeriods.at(i) < 1.01*-1))
		{
			indexOfZeroFrequency = i;
			zeroFrequencyAddedMass = addedMass.at(indexOfZeroFrequency);
		}
	}
	//Matrix<double,6,6> temp = zeroFrequencyAddedMass;
	//Matrix<double,6,1> diagVec = zeroFrequencyAddedMass.diagonal();
	//zeroFrequencyAddedMass = diagVec.asDiagonal();

	if(indexOfZeroFrequency < 0){
		std::cout << "WAMIT has to be run with infinity as one of the periods. (PER = -1)" << std::endl;
		zeroFrequencyAddedMass = addedMass.back();
	}

	potentialA = addedMass;
	potentialB = potentialDamping;

	mInv = (rigidBodyMassMatrix+zeroFrequencyAddedMass).inverse();

}

void WamitVessel::getViscousForces(const double T,const double* const X){

	const double* viscousForceArray = viscousForce->getViscousForces(X+indexNu);

	for (size_t i = 0; i<6 ; i++){
		v_viscousForces(i) = viscousForceArray[i];
	}
	
}