////////////////////////////////////////////////////////////////////////////////////////////////////
/// \class							WaveDriftWamit
////////////////////////////////////////////////////////////////////////////////////////////////////
//
/// \brief			Reads the wavedrift force RAO's from WAMIT results from pressure integration
///
///					*.9			Mean drift forces and moments from pressure
///
///					
/// \author			Mats Nvik Hval
///
///	\par
///					08.03.2012 MNH: Initial version
///
////////////////////////////////////////////////////////////////////////////////////////////////////
#include "WaveDriftWamit.h"
#include <math.h>

using namespace std;
using namespace Eigen;

//void WaveDriftWamit::TokenizeString(const string &line,vector<string> &tokens)
//{
//	tokens.clear();
//	size_t lastPos = line.find_first_not_of(" ",0);
//	size_t pos = line.find_first_of(" ",lastPos);
//
//	while (string::npos != pos || string::npos !=lastPos){
//		tokens.push_back(line.substr(lastPos,pos-lastPos));
//		lastPos = line.find_first_not_of(" ",pos);
//		pos = line.find_first_of(" ",lastPos);
//	}
//}
//void WaveDriftWamit::DiscreteToContinousScaling(const double minX, const double maxX, const double currentValue, double *alpha1, double *alpha2)
//{
//	/// Based on first-order-hold 
//
//	*alpha1 = (currentValue - minX) / (maxX - minX);
//	*alpha2 = 1- *alpha1;
//}
//
//
//
//void WaveDriftWamit::FindIndices(unsigned int *indexBelow, unsigned int *indexAbove, double value, std::vector<double> vectorOfValues,const unsigned int indexOfLastBelow, const unsigned int indexOfLastAbove)
//{
//	unsigned int upperIndex = 0;
//	unsigned int lowerIndex = 0;
//	unsigned int oneBelowPreviousValue = (indexOfLastBelow-1 >1)?indexOfLastBelow-1:1;
//	unsigned int twoBelowPreviousValue = (indexOfLastBelow-2 >1)?indexOfLastBelow-2:1;
//	unsigned int oneAbovePreviousValue = (indexOfLastAbove+1 >1)?indexOfLastAbove+1:1;
//	unsigned int twoAbovePreviousValue = (indexOfLastAbove+2 >1)?indexOfLastAbove+2:1;
//	
//	if (value < wavePeriod[indexOfLastAbove] && value >= wavePeriod[indexOfLastBelow])
//	{
//		upperIndex = indexOfLastAbove;
//		lowerIndex = indexOfLastBelow;
//	}
//	else if ( value < wavePeriod[oneAbovePreviousValue] && value >= wavePeriod[indexOfLastAbove])
//		{
//			upperIndex = oneAbovePreviousValue;
//			lowerIndex = indexOfLastAbove;
//		}
//	else if ( value < wavePeriod[twoAbovePreviousValue] && value >= wavePeriod[oneAbovePreviousValue])
//		{
//			upperIndex = twoAbovePreviousValue;
//			lowerIndex = oneAbovePreviousValue;
//		}
//	else if ( value < wavePeriod[indexOfLastWavePeriodBelow] && value >= wavePeriod[oneBelowPreviousValue])
//		{
//			upperIndex = indexOfLastWavePeriodBelow;
//			lowerIndex = oneBelowPreviousValue;
//		}
//	else if ( value < wavePeriod[oneBelowPreviousValue] && value >= wavePeriod[twoBelowPreviousValue])
//		{
//			upperIndex = oneBelowPreviousValue;
//			lowerIndex = twoBelowPreviousValue;
//		}
//	else 
//	{
//	
//		//unsigned int upperIndex = vectorOfValues.size()-1;
//		//unsigned int lowerIndex = 1;
//
//		if (value == vectorOfValues.back()) //(length(wavePeriod)))
//		{
//			upperIndex = vectorOfValues.size()-1;
//			lowerIndex = vectorOfValues.size()-1;
//		}
//		else if (value == wavePeriod[1])
//		{
//			upperIndex = 1;
//			lowerIndex = 1;
//		}
//		else if (value < vectorOfValues.back() && value>0)
//		{
//			while (upperIndex-lowerIndex > 1 && value >= vectorOfValues[lowerIndex] && value <= vectorOfValues[upperIndex])
//			{
//				unsigned int splitterIndex = min(lowerIndex,upperIndex) + (upperIndex - lowerIndex) /2;
//				if ( value > vectorOfValues[splitterIndex])
//					lowerIndex = splitterIndex;
//				if ( value < vectorOfValues[splitterIndex])
//					upperIndex = splitterIndex;
//				if (upperIndex - lowerIndex == 2 && value == vectorOfValues[lowerIndex+1])
//				{
//					upperIndex = lowerIndex+1;
//					lowerIndex = lowerIndex+1;
//				}
//			}
//		}
//		else
//		{
//			lowerIndex = vectorOfValues.size()-1;
//			upperIndex = 0;
//		}
//	
//		* indexAbove = upperIndex;
//		* indexBelow = lowerIndex;
//	}
//}


void WaveDriftWamit::Read( const std::string &fname )
	{
	ifstream file;				
	file.open(fname + ".out");
	string line;
	vector<string> tokens;
	while ( file.good() ){
		line.clear();
		getline(file,line);
		
		if(line.find("Gravity:",0) != -1){
			TokenizeString(line,tokens);
			GRAVITY = float(atof(tokens.at(1).c_str()));
			ULEN = float(atof(tokens.at(4).c_str()));
		}
		if(line.find("Water depth",0) != -1){
			TokenizeString(line,tokens);
			WATERDEPTH = float(atof(tokens.at(2).c_str()));
			RHO = float(atof(tokens.at(5).c_str()));
		}
		tokens.clear();
	}
	file.close();
	
	file.open(fname + ".9");
	vector<double> floatTokens; // vector containing the values in current line
	
	//vector< vector< vector< vector< Eigen::Matrix<double,9,4> > > > > RAOdata;
	vector<Eigen::Matrix<double,9,4> >temp1;
	//vector< Eigen::Matrix<double,9,4> > temp2;
	//vector< Eigen::Matrix<double,9,4> > temp3;
	RAOdata.push_back(temp1);
	//RAOdata.back().push_back(temp2);
	Matrix<double,9,4> meanDriftDataContent;
	meanDriftDataContent.setZero();

	while ( file.good() ){
		line.clear();
		tokens.clear();
		floatTokens.clear();
		getline(file,line);
		TokenizeString(line,tokens);
			
		
		//vector<double> wavePeriod;
		//vector<double> waveHeading;
		//vector<double> secondWaveHeading;
	
		if( !tokens.empty()) {
			for (unsigned int i=0;i<tokens.size();i++) floatTokens.push_back(float(atof(tokens[i].c_str())));
			if (int(floatTokens[3]) == 1)
			{
				meanDriftDataContent.setZero();
				/*if (floatTokens[2] != secondWaveHeading.back())
				{
					secondWaveHeading.push_back(floatTokens[2]);
				}*/
				if  (floatTokens[2] != wavePeriod.back()) //(periods.empty() || periods.back() != floatTokens.at(0)) 
				{
					wavePeriod.push_back(floatTokens[0]);	
				}
				if  (floatTokens[2] != waveHeading.back()) //(waveHeadings.empty() || waveHeadings.back() != floatTokens.at(1)) 
				{
					waveHeading.push_back(floatTokens[1]);
					RAOdata.push_back(temp1);
				}
				RAOdata.back().push_back(meanDriftDataContent);
			
			}
			int dof = int(floatTokens[3]);
			if (dof > 0 && dof <4){
				RAOdata.back().back()(dof-1,0) = floatTokens.at(4)*GRAVITY*RHO*ULEN;
				RAOdata.back().back()(dof-1,1) = floatTokens.at(5); 
				RAOdata.back().back()(dof-1,2) = floatTokens.at(6)*GRAVITY*RHO*ULEN; 
				RAOdata.back().back()(dof-1,3) = floatTokens.at(7)*GRAVITY*RHO*ULEN; 
			}
			if ( dof >3){	//surge and sway on the output file
					RAOdata.back().back()(dof-1,0) = floatTokens.at(4)*GRAVITY*RHO*pow(ULEN,2); 
					RAOdata.back().back()(dof-1,1) = floatTokens.at(5); 
					RAOdata.back().back()(dof-1,2) = floatTokens.at(6)*GRAVITY*RHO*pow(ULEN,2); 
					RAOdata.back().back()(dof-1,3) = floatTokens.at(7)*GRAVITY*RHO*pow(ULEN,2); 
				}
		}
	}
	file.close();
	indexOfLastWaveHeadingAbove = 0;
	indexOfLastWaveHeadingBelow = 0;
	indexOfLastWavePeriodAbove = 1;
	indexOfLastWavePeriodBelow = 1;
}

std::complex<double> WaveDriftWamit::GetRAOValue(int DOF, double ShipSpeed, double WaveFrequency, double WaveDirection)
{
	/// Find index for the period below and above the current period (frequency) in the wavePeriod vector
	
	double dWavePeriod = (WaveFrequency != 0)?1/WaveFrequency:0;
	unsigned int indexForSmallerPeriod,indexForLargerPeriod;

	FindIndices(&indexForSmallerPeriod,&indexForLargerPeriod,dWavePeriod,wavePeriod,indexOfLastWavePeriodBelow,indexOfLastWavePeriodAbove);
	
	/// Find index for the wave direction below and above the current wave direction in the waveHeading vector
	unsigned int indexForSmallerHeading, indexForLargerHeading;
	bool largerThan180 = false;
	if (WaveDirection >180){
		WaveDirection = fmod(WaveDirection,180);
		largerThan180 = true;
		}
	FindIndices(&indexForSmallerHeading,&indexForLargerHeading,WaveDirection,waveHeading,indexOfLastWaveHeadingBelow,indexOfLastWaveHeadingAbove);

	double valuePeriodUnder = wavePeriod[indexForSmallerPeriod];
	double valuePeriodOver = wavePeriod[indexForLargerPeriod];
	
	double valueHeadingUnder = waveHeading[indexForSmallerHeading];
	double valueHeadingOver = waveHeading[indexForLargerHeading];

	/// scale RAOdata to current heading and period

	double alpha1,alpha2;
	double beta1, beta2;
	
	DiscreteToContinousScaling(wavePeriod[indexForSmallerPeriod],wavePeriod[indexForLargerPeriod],dWavePeriod,&alpha1,&alpha2);
	
	DiscreteToContinousScaling(waveHeading[indexForSmallerHeading],waveHeading[indexForLargerHeading],WaveDirection,&beta1,&beta2);

	double imPart,rePart;

	rePart = beta1*(alpha1*RAOdata[indexForSmallerHeading][indexForSmallerPeriod](DOF,3) + alpha2 * RAOdata[indexForSmallerHeading][indexForLargerPeriod](DOF,3)) + beta2*(alpha1*RAOdata[indexForLargerHeading][indexForSmallerPeriod](DOF,3) + alpha2 * RAOdata[indexForLargerHeading][indexForLargerPeriod](DOF,3));
	imPart = beta1*(alpha1*RAOdata[indexForSmallerHeading][indexForSmallerPeriod](DOF,4) + alpha2 * RAOdata[indexForSmallerHeading][indexForLargerPeriod](DOF,4)) + beta2*(alpha1*RAOdata[indexForLargerHeading][indexForSmallerPeriod](DOF,4) + alpha2 * RAOdata[indexForLargerHeading][indexForLargerPeriod](DOF,4));

	std::complex<double> RAOvalue (rePart,imPart);

	return RAOvalue;
}